import json
import subprocess
import unittest
from typing import Any, Dict

from sacrerouge.common import TemporaryDirectory
from sacrerouge.commands.stat_sig_test import run_wilcoxon_tests
from sacrerouge.common.testing.util import sacrerouge_command_exists


class TestStatisticalSignifianceTest(unittest.TestCase):
    """
    In scipy 1.4.1, the only method to calculate the p-value is through a normal approximation. In 1.5.2, they
    added an exact calculation for samples smaller than 25 by default. These unit tests test a sample size of 3, and
    therefore the change from 1.4.1 to 1.5.2 results in a different p-value. The tests here are written
    to check the results against scipy 1.5.2.
    """
    def setUp(self) -> None:
        self.correlations_A = {
            'pearson': {'A': 0.72, 'B': 0.14, 'C': 0.84, 'D': 0.98, 'E': 0.88},
            'spearman': {'A': 0.04, 'B': 0.32, 'C': 0.54, 'D': 0.18},
        }
        self.correlations_B = {
            'pearson': {'A': 0.24, 'B': 0.75, 'C': 0.34, 'D': 0.34, 'E': 0.22},
            'spearman': {'A': 0.98, 'B': 0.86, 'C': 0.77, 'E': 0.53},
            'kendall': {'A': 0.34, 'B': 0.88}
        }

    def _verify_results(self, results: Dict[str, Any]):
        assert results['alternative'] == 'two-sided'
        assert results['pearson']['n'] == 5
        assert results['pearson']['statistic'] == 3.0
        assert results['pearson']['p_value'] == 0.3125

        assert results['spearman']['n'] == 3
        assert results['spearman']['statistic'] == 0.0
        assert results['spearman']['p_value'] == 0.25

        assert 'kendall' not in results

    def test_command_exists(self):
        assert sacrerouge_command_exists(['stat-sig-test'])

    def test_run_wilcoxon_tests(self):
        # This is a regression test and does not test for accuracy
        results = run_wilcoxon_tests(self.correlations_A, self.correlations_B)
        self._verify_results(results)

    def test_command_line(self):
        # This is a regression test and does not test for accuracy
        with TemporaryDirectory() as temp_dir:
            with open(f'{temp_dir}/A.json', 'w') as out:
                out.write(json.dumps(self.correlations_A))
            with open(f'{temp_dir}/B.json', 'w') as out:
                out.write(json.dumps(self.correlations_B))

            command = [
                'python', '-m', 'sacrerouge', 'stat-sig-test',
                '--summary-level-correlations-A', f'{temp_dir}/A.json',
                '--summary-level-correlations-B', f'{temp_dir}/B.json',
                '--output-file', f'{temp_dir}/results.json',
                '--silent'
            ]
            subprocess.run(command, check=True)
            results = json.load(open(f'{temp_dir}/results.json', 'r'))
            self._verify_results(results)


import argparse
import json
import lxml.html
import re
import tarfile
from collections import defaultdict
from io import BytesIO
from nltk.tokenize import sent_tokenize
from typing import Any, Dict, List

from sacrerouge.io import JsonlWriter


def parse_document(html: str) -> List[str]:
    tree = lxml.html.document_fromstring(html)
    nodes = list(tree.xpath('//text'))
    if not nodes:
        nodes = list(tree.xpath('//graphic'))
    root = nodes[0]
    text = root.text_content().strip()
    text = re.sub(r'\s+', ' ', text)
    sentences = sent_tokenize(text)
    return sentences


def load_task1_documents(documents_tar: str):
    print('Loading Task 1 Documents')
    cluster_to_doc_ids, documents = defaultdict(list), {}
    with tarfile.open(documents_tar, 'r') as tar:
        inner_tar_bytes = tar.extractfile('DUC2003_Summarization_Documents/duc2003_testdata/task1/task1.docs.tar.gz').read()
        with tarfile.open(fileobj=BytesIO(inner_tar_bytes)) as inner_tar:
            for member in inner_tar.getmembers():
                if member.isfile():
                    _, _, cluster, doc_id = member.name.split('/')
                    cluster_to_doc_ids[cluster].append(doc_id)

                    text = inner_tar.extractfile(member).read().decode()
                    sentences = parse_document(text)
                    document = {
                        'filename': doc_id,
                        'text': sentences
                    }
                    if doc_id in documents:
                        assert documents[doc_id] == document
                    documents[doc_id] = document
                    print(f'Loaded document {doc_id} in cluster {cluster}')
    return cluster_to_doc_ids, documents


def load_task2_documents(documents_tar: str):
    cluster_to_doc_ids, documents = defaultdict(list), {}
    with tarfile.open(documents_tar, 'r') as tar:
        inner_tar_bytes = tar.extractfile('DUC2003_Summarization_Documents/duc2003_testdata/task2/task2.docs.tar.gz').read()
        with tarfile.open(fileobj=BytesIO(inner_tar_bytes)) as inner_tar:
            for member in inner_tar.getmembers():
                if member.isfile():
                    _, _, cluster, doc_id = member.name.split('/')
                    cluster = cluster[:-1]
                    cluster_to_doc_ids[cluster].append(doc_id)

                    text = inner_tar.extractfile(member).read().decode()
                    sentences = parse_document(text)
                    document = {
                        'filename': doc_id,
                        'text': sentences
                    }
                    if doc_id in documents:
                        assert documents[doc_id] == document
                    documents[doc_id] = document
    return cluster_to_doc_ids, documents


def load_task2_topics(documents_tar: str):
    metadata = json.loads(open('datasets/duc-tac/duc2003/topics-metadata.json', 'r').read())
    with tarfile.open(documents_tar, 'r') as tar:
        text = tar.extractfile('DUC2003_Summarization_Documents/duc2003_testdata/task2/topics').read().decode()

    topics = {}
    for info in metadata:
        cluster = 'd' + info['cluster']

        start, end = info['title']
        title = text[start:end]

        event = {}
        for wh, (start, end) in info['event'].items():
            event[wh] = re.sub(f'\s+', ' ', text[start:end])

        start, end = info['explication']
        explication = sent_tokenize(re.sub(f'\s+', ' ', text[start:end]))

        topics[cluster] = {
            'title': title,
            'event': event,
            'explication': explication
        }
    return topics


def load_task3_documents(documents_tar: str):
    cluster_to_doc_ids, documents = defaultdict(list), {}
    with tarfile.open(documents_tar, 'r') as tar:
        inner_tar_bytes = tar.extractfile('DUC2003_Summarization_Documents/duc2003_testdata/task3/task3.docs.tar.gz').read()
        with tarfile.open(fileobj=BytesIO(inner_tar_bytes)) as inner_tar:
            for member in inner_tar.getmembers():
                if member.isfile():
                    _, _, cluster, doc_id = member.name.split('/')
                    cluster = cluster[:-1]
                    cluster_to_doc_ids[cluster].append(doc_id)

                    text = inner_tar.extractfile(member).read().decode()
                    sentences = parse_document(text)
                    document = {
                        'filename': doc_id,
                        'text': sentences
                    }
                    if doc_id in documents:
                        assert documents[doc_id] == document
                    documents[doc_id] = document
    return cluster_to_doc_ids, documents


def load_task3_viewpoints(documents_tar: str) -> Dict[str, str]:
    viewpoints = {}
    with tarfile.open(documents_tar, 'r') as tar:
        inner_tar_bytes = tar.extractfile('DUC2003_Summarization_Documents/duc2003_testdata/task3/viewpoints.given.to.systems.tar.gz').read()
        with tarfile.open(fileobj=BytesIO(inner_tar_bytes)) as inner_tar:
            for member in inner_tar.getmembers():
                _, _, cluster, _ = member.name.split('/')
                cluster = cluster[:-2]

                lines = inner_tar.extractfile(member).read().decode().splitlines()
                lines = list(filter(None, lines))
                viewpoint = []
                for line in lines:
                    if (line.startswith('<VIEWPOINT') or line.startswith('DOCSET=') or
                       line.startswith('CREATOR=') or line.startswith('</VIEWPOINT>')):
                        continue
                    viewpoint.append(line.strip())
                viewpoint = ' '.join(viewpoint)
                viewpoints[cluster] = viewpoint
    return viewpoints


def load_task4_abstractive_documents(documents_tar: str):
    cluster_to_doc_ids, documents = defaultdict(list), {}
    with tarfile.open(documents_tar, 'r') as tar:
        inner_tar_bytes = tar.extractfile('DUC2003_Summarization_Documents/duc2003_testdata/task4/task4.docs.tar.gz').read()
        with tarfile.open(fileobj=BytesIO(inner_tar_bytes)) as inner_tar:
            for member in inner_tar.getmembers():
                _, _, cluster, doc_id = member.name.split('/')
                cluster = cluster[:-1]
                cluster_to_doc_ids[cluster].append(doc_id)

                text = inner_tar.extractfile(member).read().decode(errors='ignore')
                sentences = parse_document(text)
                document = {
                    'filename': doc_id,
                    'text': sentences
                }
                if doc_id in documents:
                    assert documents[doc_id] == document
                documents[doc_id] = document
    return cluster_to_doc_ids, documents


def load_task4_extractive_documents(documents_tar: str):
    cluster_to_doc_ids, documents = defaultdict(set), defaultdict(list)
    with tarfile.open(documents_tar, 'r') as tar:
        for member in tar.getmembers():
            if member.isfile() and member.name.startswith('DUC2003_Summarization_Documents/trec2002_novelty_docs'):
                path = member.name.split('/')
                cluster = 'd' + path[-1][:3]

                html = tar.extractfile(member).read().decode(errors='replace')
                tree = lxml.html.document_fromstring(html)
                for doc_node in tree.xpath('//doc'):
                    sentences = []
                    # //s from doc_node retrieves all the sentences across
                    # all of the documents. Not sure why
                    for sent_node in doc_node.iter('s'):
                        doc_id = sent_node.get('docid')
                        sentence = sent_node.text_content().strip()
                        sentences.append(sentence)
                        cluster_to_doc_ids[cluster].add(doc_id)
                    document = {
                        'filename': doc_id,
                        'text': sentences
                    }
                    documents[doc_id] = document
    return cluster_to_doc_ids, documents


def load_task4_sentence_labels(documents_tar: str):
    summaries = {}
    with tarfile.open(documents_tar, 'r') as tar:
        inner_tar_bytes = tar.extractfile('DUC2003_Summarization_Documents/duc2003_testdata/task4/task4.sentences.tar.gz').read()
        with tarfile.open(fileobj=BytesIO(inner_tar_bytes)) as inner_tar:
            for member in inner_tar.getmembers():
                _, _, filename = member.name.split('/')
                cluster = 'd' + filename[:3]
                html = inner_tar.extractfile(member).read().decode(errors='ignore')
                tree = lxml.html.document_fromstring(html)
                labels = []
                for node in tree.xpath('//s'):
                    doc_id = node.get('docid')
                    num = int(node.get('num')) - 1
                    is_novel = 'novel' in node.attrib
                    labels.append({
                        'filename': doc_id,
                        'sent_index': num,
                        'is_novel': is_novel
                    })
                summaries[cluster] = labels
    return summaries


def split_into_tags(lines: List[str]) -> List[List[str]]:
    tags = []
    current = []
    for line in lines:
        line = line.strip()
        if not line:
            continue
        if line.startswith('<'):
            # New tag
            if current:
                tags.append(current)
            current = [line]
        else:
            current.append(line)
    if current:
        tags.append(current)
    return tags


def group_by_clusters(tags: List[List[str]]) -> List[List[List[str]]]:
    groups = []
    current = []
    for tag in tags:
        if tag == ['<top>']:
            if current:
                groups.append(current)
            current = []
        else:
            current.append(tag)
    if current:
        groups.append(current)
    return groups


def load_task4_questions(documents_tar: str):
    with tarfile.open(documents_tar, 'r') as tar:
        lines = tar.extractfile('DUC2003_Summarization_Documents/duc2003_testdata/task4/duc.novelty_topics.txt').read().decode().splitlines()

    tags = split_into_tags(lines)
    groups = group_by_clusters(tags)

    questions = {}
    for group in groups:
        assert group[0][0].startswith('<num>')
        assert group[1][0].startswith('<title>')
        assert group[2][0].startswith('<desc>')
        assert group[3][0].startswith('<desc2>')
        assert group[4][0].startswith('<narr>')

        cluster = 'd' + group[0][0][len('<num> Number: '):]
        title = group[1][0][len('<title> '):]
        desc1 = ' '.join(group[2][1:])
        desc2 = ' '.join(group[3][1:])
        narr = sent_tokenize(' '.join(group[4][1:]))
        assert cluster not in questions, cluster
        questions[cluster] = {
            'title': title,
            'description1': desc1,
            'description2': desc2,
            'narrative': narr
        }
    return questions


def load_summaries(summaries_tar: str):
    sds, mds = defaultdict(list), defaultdict(list)

    with tarfile.open(summaries_tar, 'r') as tar:
        for member in tar.getmembers():
            if member.isfile():
                sentences = tar.extractfile(member).read().decode().strip().splitlines()
                sentences = ' '.join(list(map(lambda line: line.strip(), sentences)))
                sentences = sent_tokenize(sentences)

                path = member.name.split('/')
                filename = path[-1]
                sections = filename.split('.')
                annotator = sections[4]
                summary = {
                    'annotator': annotator,
                    'text': sentences
                }

                is_multi = sections[1] == 'M'
                if is_multi:
                    assert len(sections) == 5
                    cluster = sections[0].lower()
                    if summary not in mds[cluster]:
                        mds[cluster].append(summary)
                    else:
                        print(f'Cluster {cluster} has duplicate summaries')
                else:
                    assert len(sections) == 7
                    doc_id = '.'.join(sections[5:7])
                    assert summary not in sds[doc_id]
                    sds[doc_id].append(summary)
    return sds, mds


def save_task1_data(documents: Dict[str, List[str]],
                    summaries: Dict[str, List[List[str]]],
                    file_path: str) -> None:
    with JsonlWriter(file_path) as out:
        for doc_id, document in documents.items():
            doc_summaries = summaries[doc_id]
            data = {
                'instance_id': doc_id,
                'document': document,
                'summaries': doc_summaries
            }
            out.write(data)


def save_task2_data(cluster_to_doc_ids: Dict[str, List[str]],
                    documents: Dict[str, List[str]],
                    summaries: Dict[str, List[List[str]]],
                    topics: Dict[str, Dict[str, Any]],
                    file_path: str) -> None:
    with JsonlWriter(file_path) as out:
        for cluster in sorted(cluster_to_doc_ids.keys()):
            cluster_docs = [documents[doc_id] for doc_id in cluster_to_doc_ids[cluster]]
            cluster_summaries = summaries[cluster]
            cluster_topic = topics[cluster]
            data = {
                'instance_id': cluster,
                'topic': cluster_topic,
                'documents': cluster_docs,
                'summaries': cluster_summaries
            }
            out.write(data)


def save_task3_data(cluster_to_doc_ids: Dict[str, List[str]],
                    documents: Dict[str, List[str]],
                    summaries: Dict[str, List[List[str]]],
                    viewpoints: Dict[str, str],
                    file_path: str) -> None:
    with JsonlWriter(file_path) as out:
        for cluster in sorted(cluster_to_doc_ids.keys()):
            cluster_docs = [documents[doc_id] for doc_id in cluster_to_doc_ids[cluster]]
            cluster_summaries = summaries[cluster]
            cluster_viewpoint = viewpoints[cluster]
            data = {
                'instance_id': cluster,
                'viewpoint': cluster_viewpoint,
                'documents': cluster_docs,
                'summaries': cluster_summaries
            }
            out.write(data)


def save_task4_abs_data(cluster_to_doc_ids: Dict[str, List[str]],
                        documents: Dict[str, List[str]],
                        summaries: Dict[str, List[List[str]]],
                        questions: Dict[str, Dict[str, Any]],
                        file_path: str) -> None:
    with JsonlWriter(file_path) as out:
        for cluster in sorted(cluster_to_doc_ids.keys()):
            cluster_docs = [documents[doc_id] for doc_id in cluster_to_doc_ids[cluster]]
            cluster_summaries = summaries[cluster]
            cluster_question = questions[cluster]
            assert len(cluster_summaries) == 4
            data = {
                'instance_id': cluster,
                'question': cluster_question,
                'documents': cluster_docs,
                'summaries': cluster_summaries
            }
            out.write(data)


def save_task4_ext_data(cluster_to_doc_ids: Dict[str, List[str]],
                        documents: Dict[str, List[str]],
                        summaries: Dict[str, List[List[Dict[str, Any]]]],
                        questions: Dict[str, Dict[str, Any]],
                        sentence_labels: Dict[str, List[Dict[str, Any]]],
                        file_path: str) -> None:
    with JsonlWriter(file_path) as out:
        for cluster in sorted(sentence_labels.keys()):
            doc_ids = list(cluster_to_doc_ids[cluster])
            doc_id_to_index = {doc_id: i for i, doc_id in enumerate(doc_ids)}

            cluster_docs = [documents[doc_id] for doc_id in doc_ids]
            cluster_summaries = summaries[cluster]
            cluster_question = questions[cluster]
            cluster_labels = sentence_labels[cluster]

            new_labels = []
            for label in cluster_labels:
                new_labels.append({
                    'doc_index': doc_id_to_index[label['filename']],
                    'sent_index': label['sent_index'],
                    'is_novel': label['is_novel']
                })

            data = {
                'instance_id': cluster,
                'question': cluster_question,
                'documents': cluster_docs,
                'summaries': cluster_summaries,
                'relevance_labels': new_labels
            }
            out.write(data)


def setup(data_root: str, output_dir: str) -> None:
    documents_tar = f'{data_root}/from-nist/DUC2003_Summarization_Documents.tgz'
    summaries_tar = f'{data_root}/scrapes/duc.nist.gov/past_duc/duc2003/results/manualsummaries/duc2003.manual.abstracts.tar.gz'
    main(documents_tar, summaries_tar, output_dir)


def main(documents_tar, summaries_tar, output_dir):
    sds, mds = load_summaries(summaries_tar)

    cluster_to_doc_ids, documents = load_task1_documents(documents_tar)
    save_task1_data(documents, sds, f'{output_dir}/task1.jsonl')

    cluster_to_doc_ids, documents = load_task2_documents(documents_tar)
    topics = load_task2_topics(documents_tar)
    save_task2_data(cluster_to_doc_ids, documents, mds, topics, f'{output_dir}/task2.jsonl')

    cluster_to_doc_ids, documents = load_task3_documents(documents_tar)
    viewpoints = load_task3_viewpoints(documents_tar)
    save_task3_data(cluster_to_doc_ids, documents, mds, viewpoints, f'{output_dir}/task3.jsonl')

    questions = load_task4_questions(documents_tar)
    cluster_to_doc_ids, abs_documents = load_task4_abstractive_documents(documents_tar)
    save_task4_abs_data(cluster_to_doc_ids, abs_documents, mds, questions, f'{output_dir}/task4.jsonl')

    cluster_to_doc_ids, ext_documents = load_task4_extractive_documents(documents_tar)
    sentence_labels = load_task4_sentence_labels(documents_tar)
    save_task4_ext_data(cluster_to_doc_ids, ext_documents, mds, questions, sentence_labels, f'{output_dir}/task4.sentence-labels.jsonl')


if __name__ == '__main__':
    argp = argparse.ArgumentParser()
    argp.add_argument('documents_tar')
    argp.add_argument('summaries_tar')
    argp.add_argument('output_dir')
    args = argp.parse_args()
    main(args)


import os
import pytest
import unittest
from collections import defaultdict
from typing import Dict, List

from sacrerouge.data import Metrics
from sacrerouge.io import JsonlReader

_metrics_file_path = 'datasets/duc-tac/duc2006/v1.0/task1.metrics.jsonl'


class TestDUC2006Metrics(unittest.TestCase):
    def _convert_to_dicts(self, metrics_list: List[Metrics]) -> Dict[str, Dict[str, Metrics]]:
        metrics_dicts = defaultdict(dict)
        for metrics in metrics_list:
            metrics_dicts[metrics.instance_id][metrics.summarizer_id] = metrics.metrics
        return metrics_dicts

    @pytest.mark.skipif(not os.path.exists(_metrics_file_path), reason='DUC 2006 metrics file does not exist')
    def test_system_level(self):
        summary_level_metrics = JsonlReader(_metrics_file_path, Metrics).read()
        summary_level_metrics = self._convert_to_dicts(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files
        # DUC2006pyramiddata/scoring/2006_modified_scores.txt
        assert summary_level_metrics['d0603']['32']['modified_pyramid_score'] == pytest.approx(0.1048, 1e-2)
        assert summary_level_metrics['d0616']['10']['modified_pyramid_score'] == pytest.approx(0.3103, 1e-2)
        assert summary_level_metrics['d0628']['6']['modified_pyramid_score'] == pytest.approx(0.0769, 1e-2)

        assert summary_level_metrics['d0603']['32']['num_scus'] == 6
        assert summary_level_metrics['d0616']['10']['num_scus'] == 7
        assert summary_level_metrics['d0628']['6']['num_scus'] == 2

        assert summary_level_metrics['d0603']['32']['num_repetitions'] == 0
        assert summary_level_metrics['d0616']['10']['num_repetitions'] == 0
        assert summary_level_metrics['d0628']['6']['num_repetitions'] == 0

        assert summary_level_metrics['d0603']['32']['content_responsiveness'] == 3
        assert summary_level_metrics['d0616']['10']['content_responsiveness'] == 4
        assert summary_level_metrics['d0628']['6']['content_responsiveness'] == 2



import argparse
from collections import defaultdict
from typing import Dict, List

from sacrerouge import build_argument_parser
from sacrerouge.data import Metrics, MetricsDict
from sacrerouge.data.types import ReferenceType, SummaryType
from sacrerouge.io import JsonlReader


def load_summaries(file_path: str) -> List[SummaryType]:
    fields = []
    for data in JsonlReader(file_path).read():
        fields.append(data['summary'])
    return fields


def load_references(file_path: str) -> List[ReferenceType]:
    fields = []
    for data in JsonlReader(file_path).read():
        if 'summary' in data:
            fields.append([data['summary']['text']])
        elif 'summaries' in data:
            fields.append([summary['text'] for summary in data['summaries']])
        elif 'reference' in data:
            fields.append([data['reference']['text']])
        elif 'references' in data:
            fields.append([reference['text'] for reference in data['references']])
    return fields


def load_metrics_dicts(file_path: str) -> Dict[str, Dict[str, MetricsDict]]:
    metrics_dicts = defaultdict(dict)
    with JsonlReader(file_path, Metrics) as f:
        for instance in f:
            metrics_dicts[instance.instance_id][instance.summarizer_id] = instance.metrics
    return metrics_dicts


def command_exists(parser: argparse.ArgumentParser, command: List[str]) -> bool:
    """
    Checks to see if a specific command exists in the `parser`. The `parser` should
    be the root `ArgumentParser` for the command. The method will traverse through
    the `parser` to see if the `command` exists. This method does not work for checking
    arguments of a specific command.
    """
    # _subparsers is none when no subcommands exist
    if parser._subparsers is None:
        return False

    for action in parser._subparsers._group_actions:
        for choice, subparser in action.choices.items():
            if choice == command[0]:
                if len(command) == 1:
                    # The whole command has been matched
                    return True
                else:
                    return command_exists(subparser, command[1:])

    # We didn't find the first command, so it doesn't exist
    return False


def sacrerouge_command_exists(command: List[str]) -> bool:
    """Verifies if the command exists for the 'sacrerouge' command."""
    parser = build_argument_parser()
    return command_exists(parser, command)


import json
import os
import pytest

from sacrerouge.common.testing import FIXTURES_ROOT
from sacrerouge.common.testing.metric_test_cases import DocumentBasedMetricTestCase
from sacrerouge.common.testing.util import sacrerouge_command_exists
from sacrerouge.metrics import SUPERT

_TOPIC_1 = f'{FIXTURES_ROOT}/data/supert/topic_1.json'


@pytest.mark.skipif('SUPERT_ENV' not in os.environ, reason='SUPERT python environment environment variable not set')
class TestSUPERT(DocumentBasedMetricTestCase):
    def test_example(self):
        # Tests to make sure we get the same output as running the example in the original repository
        topic1 = json.load(open(_TOPIC_1, 'r'))
        documents = topic1['documents']
        summaries = topic1['summaries']

        supert = SUPERT(os.environ['SUPERT_ENV'])
        expected_output = [
            {'supert': 0.4724942240404387},
            {'supert': 0.38462300158000995},
            {'supert': 0.4947284743279349},
            {'supert': 0.501369657633312},
            {'supert': 0.5108337561530968}
        ]
        actual_output = supert.score_multi(summaries, documents)

        for expected, actual in zip(expected_output, actual_output):
            assert expected == pytest.approx(actual, 1e-4)

    def test_supert(self):
        # This is a regression test, not necessarily a test for correctness
        metric = SUPERT(os.environ['SUPERT_ENV'])
        expected_output = [
            {'supert': 0.46000519191903655},
            {'supert': 0.4766808595557026},
            {'supert': 0.4467075912019479},
            {'supert': 0.4049276967180207},
            {'supert': 0.4518371118994575},
            {'supert': 0.4672364921295776},
            {'supert': 0.4413289374922482},
            {'supert': 0.41049172025776465},
            {'supert': 0.5598548871327522},
            {'supert': 0.5688464141370209},
            {'supert': 0.5096871293947829},
            {'supert': 0.4635047957553205}
        ]
        super().assert_expected_output(metric, expected_output)

    def test_supert_order_invariant(self):
        metric = SUPERT(os.environ['SUPERT_ENV'])
        self.assert_order_invariant(metric)

    def test_command_exists(self):
        assert sacrerouge_command_exists(['supert'])

    def test_setup_command_exists(self):
        assert sacrerouge_command_exists(['setup-metric', 'supert'])


import logging
from typing import Any, List, Union

from sacrerouge.data import EvalInstance
from sacrerouge.data.dataset_readers import DatasetReader
from sacrerouge.data.fields import DocumentsField, Fields, SummaryField
from sacrerouge.io import JsonlReader

logger = logging.getLogger(__name__)


def flatten_document(document: Any) -> Union[List[str], str]:
    """"
    If the document is a single string, nothing is changed. If the document is a nested list of strings,
    it will be flattened to be one list of strings without any nesting.
    """
    if isinstance(document, str):
        return document
    flat = []
    for item in document:
        if isinstance(item, str):
            flat.append(item)
        else:
            flat.extend(flatten_document(item))
    return flat


def flatten_documents(documents: List[Any]) -> List[Union[List[str], str]]:
    return [flatten_document(document) for document in documents]


@DatasetReader.register('document-based')
class DocumentBasedDatasetReader(DatasetReader):
    def read(self, input_jsonl: str) -> List[EvalInstance]:
        logger.info(f'Loading evaluation instances from {input_jsonl}')
        instances = []
        with JsonlReader(input_jsonl) as f:
            for data in f:
                fields = {}
                fields['summary'] = SummaryField(data['summary']['text'])

                if 'document' in data:
                    fields['documents'] = DocumentsField([data['document']['text']])
                else:
                    fields['documents'] = DocumentsField([document['text'] for document in data['documents']])
                fields = Fields(fields)

                instance = EvalInstance(
                    data['instance_id'],
                    data['summarizer_id'],
                    data['summarizer_type'],
                    fields
                )
                instances.append(instance)

            logger.info(f'Loaded {len(instances)} instances')
            return instances


@DatasetReader.register('split-document-based')
class SplitDocumentBasedDatasetReader(DatasetReader):
    def read(self, documents_jsonl: str, summaries_jsonl) -> List[EvalInstance]:
        logger.info(f'Loading documents from {documents_jsonl}')
        documents_dict = {}
        with JsonlReader(documents_jsonl) as f:
            for data in f:
                instance_id = data['instance_id']
                if 'document' in data:
                    documents = [data['document']['text']]
                else:
                    documents = [document['text'] for document in data['documents']]
                documents = flatten_documents(documents)
                documents_dict[instance_id] = DocumentsField(documents)
        logger.info(f'Loaded {len(documents_dict)} document sets')

        logger.info(f'Loading summaries from {summaries_jsonl}')
        instances = []
        with JsonlReader(summaries_jsonl) as f:
            for data in f:
                fields = {}
                fields['summary'] = SummaryField(data['summary']['text'])

                instance_id = data['instance_id']
                fields['documents'] = documents_dict[instance_id]
                fields = Fields(fields)

                instance = EvalInstance(
                    data['instance_id'],
                    data['summarizer_id'],
                    data['summarizer_type'],
                    fields
                )
                instances.append(instance)
        logger.info(f'Loaded {len(instances)} instances')
        return instances



from sacrerouge.data.dataset_readers.dataset_reader import DatasetReader
from sacrerouge.data.dataset_readers.document_based import DocumentBasedDatasetReader, SplitDocumentBasedDatasetReader
from sacrerouge.data.dataset_readers.pyramid_based import PyramidBasedDatasetReader
from sacrerouge.data.dataset_readers.reference_based import ReferenceBasedDatasetReader
from sacrerouge.data.dataset_readers.summary_only import SummaryOnlyDatasetReader



from overrides import overrides
from typing import List

from sacrerouge.data import MetricsDict
from sacrerouge.data.jackknifers import ReferencesJackknifer
from sacrerouge.data.types import ReferenceType, SummaryType
from sacrerouge.metrics import Metric, ReferenceBasedMetric


@Metric.register('testing')
class TestingMetric(ReferenceBasedMetric):
    def __init__(self) -> None:
        super().__init__(['summary'], ['references'], jackknifer=ReferencesJackknifer())

    @overrides
    def score_multi_all(self,
                        summaries_list: List[List[SummaryType]],
                        references_list: List[List[ReferenceType]]) -> List[List[MetricsDict]]:
        metrics_dict_lists = []
        for summaries, references in zip(summaries_list, references_list):
            metrics_dict_lists.append([])
            for summary in summaries:
                summary_value = float(summary)
                total = 0
                for reference in references:
                    reference_value = float(reference)
                    total += summary_value * reference_value
                metrics_dict_lists[-1].append(MetricsDict({'test': total}))
        return metrics_dict_lists



import os
import pytest
import unittest

from sacrerouge.commands.correlate import aggregate_metrics
from sacrerouge.data import Metrics
from sacrerouge.io import JsonlReader

_metrics_file_path = 'datasets/duc-tac/tac2008/v1.0/task1.A-B.metrics.jsonl'


class TestTAC2008SystemLevel(unittest.TestCase):
    @pytest.mark.skipif(not os.path.exists(_metrics_file_path), reason='TAC 2008 metrics file does not exist')
    def test_system_level(self):
        summary_level_metrics = JsonlReader(_metrics_file_path, Metrics).read()
        system_level_metrics = aggregate_metrics(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files
        # ROUGE/rouge2.m.avg
        assert system_level_metrics['43']['rouge-2']['recall'] == pytest.approx(10.382, 1e-2)
        assert system_level_metrics['13']['rouge-2']['recall'] == pytest.approx(9.900, 1e-2)
        assert system_level_metrics['14']['rouge-2']['recall'] == pytest.approx(9.773, 1e-2)
        assert system_level_metrics['2']['rouge-2']['recall'] == pytest.approx(9.610, 1e-2)
        assert system_level_metrics['65']['rouge-2']['recall'] == pytest.approx(9.558, 1e-2)

        # ROUGE/rouge2.jk.m.avg
        assert system_level_metrics['D']['rouge-2_jk']['recall'] == pytest.approx(13.197, 1e-2)
        assert system_level_metrics['F']['rouge-2_jk']['recall'] == pytest.approx(12.896, 1e-2)
        assert system_level_metrics['H']['rouge-2_jk']['recall'] == pytest.approx(12.010, 1e-2)
        assert system_level_metrics['43']['rouge-2_jk']['recall'] == pytest.approx(10.395, 1e-2)
        assert system_level_metrics['13']['rouge-2_jk']['recall'] == pytest.approx(9.901, 1e-2)

        # ROUGE/rougeSU4.m.avg
        assert system_level_metrics['43']['rouge-su4']['recall'] == pytest.approx(13.625, 1e-2)
        assert system_level_metrics['37']['rouge-su4']['recall'] == pytest.approx(13.574, 1e-2)
        assert system_level_metrics['60']['rouge-su4']['recall'] == pytest.approx(13.570, 1e-2)
        assert system_level_metrics['2']['rouge-su4']['recall'] == pytest.approx(13.419, 1e-2)
        assert system_level_metrics['14']['rouge-su4']['recall'] == pytest.approx(13.283, 1e-2)

        # ROUGE/rougeSU4.jk.m.avg
        assert system_level_metrics['D']['rouge-su4_jk']['recall'] == pytest.approx(16.878, 1e-2)
        assert system_level_metrics['F']['rouge-su4_jk']['recall'] == pytest.approx(16.490, 1e-2)
        assert system_level_metrics['H']['rouge-su4_jk']['recall'] == pytest.approx(15.565, 1e-2)
        assert system_level_metrics['43']['rouge-su4_jk']['recall'] == pytest.approx(13.646, 1e-2)
        assert system_level_metrics['37']['rouge-su4_jk']['recall'] == pytest.approx(13.592, 1e-2)

        # manual/manual.model.avg
        assert system_level_metrics['A']['num_scus_jk'] == pytest.approx(8.021, 1e-2)
        assert system_level_metrics['B']['num_scus_jk'] == pytest.approx(8.479, 1e-2)
        assert system_level_metrics['C']['num_scus_jk'] == pytest.approx(8.208, 1e-2)

        assert system_level_metrics['A']['modified_pyramid_score_jk'] == pytest.approx(0.608, 1e-2)
        assert system_level_metrics['B']['modified_pyramid_score_jk'] == pytest.approx(0.625, 1e-2)
        assert system_level_metrics['C']['modified_pyramid_score_jk'] == pytest.approx(0.651, 1e-2)

        assert system_level_metrics['A']['linguistic_quality'] == pytest.approx(4.833, 1e-2)
        assert system_level_metrics['B']['linguistic_quality'] == pytest.approx(4.812, 1e-2)
        assert system_level_metrics['C']['linguistic_quality'] == pytest.approx(4.604, 1e-2)

        assert system_level_metrics['A']['overall_responsiveness'] == pytest.approx(4.688, 1e-2)
        assert system_level_metrics['B']['overall_responsiveness'] == pytest.approx(4.583, 1e-2)
        assert system_level_metrics['C']['overall_responsiveness'] == pytest.approx(4.500, 1e-2)

        # manual/manual.peer.avg
        assert system_level_metrics['0']['modified_pyramid_score'] == pytest.approx(0.166, 1e-2)
        assert system_level_metrics['1']['modified_pyramid_score'] == pytest.approx(0.265, 1e-2)
        assert system_level_metrics['2']['modified_pyramid_score'] == pytest.approx(0.280, 1e-2)

        assert system_level_metrics['0']['num_scus'] == pytest.approx(2.635, 1e-2)
        assert system_level_metrics['1']['num_scus'] == pytest.approx(3.854, 1e-2)
        assert system_level_metrics['2']['num_scus'] == pytest.approx(4.000, 1e-2)

        assert system_level_metrics['0']['num_repetitions'] == pytest.approx(0.688, 1e-2)
        assert system_level_metrics['1']['num_repetitions'] == pytest.approx(0.885, 1e-2)
        assert system_level_metrics['2']['num_repetitions'] == pytest.approx(1.156, 1e-2)

        assert system_level_metrics['0']['modified_pyramid_score_jk'] == pytest.approx(0.163, 1e-2)
        assert system_level_metrics['1']['modified_pyramid_score_jk'] == pytest.approx(0.261, 1e-2)
        assert system_level_metrics['2']['modified_pyramid_score_jk'] == pytest.approx(0.276, 1e-2)

        assert system_level_metrics['0']['linguistic_quality'] == pytest.approx(3.333, 1e-2)
        assert system_level_metrics['1']['linguistic_quality'] == pytest.approx(2.719, 1e-2)
        assert system_level_metrics['2']['linguistic_quality'] == pytest.approx(2.354, 1e-2)

        assert system_level_metrics['0']['overall_responsiveness'] == pytest.approx(2.073, 1e-2)
        assert system_level_metrics['1']['overall_responsiveness'] == pytest.approx(2.427, 1e-2)
        assert system_level_metrics['2']['overall_responsiveness'] == pytest.approx(2.385, 1e-2)

        # BE/simple.m.hm.avg
        assert system_level_metrics['14']['rouge-be-hm']['recall'] == pytest.approx(6.462, 1e-2)
        assert system_level_metrics['65']['rouge-be-hm']['recall'] == pytest.approx(6.276, 1e-2)
        assert system_level_metrics['43']['rouge-be-hm']['recall'] == pytest.approx(6.257, 1e-2)
        assert system_level_metrics['49']['rouge-be-hm']['recall'] == pytest.approx(6.247, 1e-2)
        assert system_level_metrics['60']['rouge-be-hm']['recall'] == pytest.approx(6.198, 1e-2)

        # BE/simple.jk.m.hm.avg
        assert system_level_metrics['D']['rouge-be-hm_jk']['recall'] == pytest.approx(9.959, 1e-2)
        assert system_level_metrics['F']['rouge-be-hm_jk']['recall'] == pytest.approx(9.553, 1e-2)
        assert system_level_metrics['G']['rouge-be-hm_jk']['recall'] == pytest.approx(9.154, 1e-2)
        assert system_level_metrics['14']['rouge-be-hm_jk']['recall'] == pytest.approx(6.480, 1e-2)
        assert system_level_metrics['65']['rouge-be-hm_jk']['recall'] == pytest.approx(6.293, 1e-2)



import argparse
from overrides import overrides

from sacrerouge.datasets.fabbri2020 import setup
from sacrerouge.commands import DatasetSetupSubcommand


@DatasetSetupSubcommand.register('fabbri2020')
class Fabbri2020Subcommand(DatasetSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the Fabbri 2020 dataset'
        self.parser = parser.add_parser('fabbri2020', description=description, help=description)
        self.parser.add_argument(
            'cnn_tar',
            type=str,
            help='The path to the downloaded tar file from this link: https://drive.google.com/uc?export=download&id=0BwmD_VLjROrfTHk4NFg2SndKcjQ'
        )
        self.parser.add_argument(
            'dailymail_tar',
            type=str,
            help='The path to the downloaded tar file from this link: https://drive.google.com/uc?export=download&id=0BwmD_VLjROrfM1BxdkxVaTY2bWs'
        )
        self.parser.add_argument(
            'output_dir',
            type=str,
            help='The directory where the data should be written'
        )
        self.parser.add_argument(
            '--force',
            action='store_true',
            help='Force the raw data to be downloaded even if it already exists on file'
        )
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        setup.setup(args.cnn_tar, args.dailymail_tar, args.output_dir, args.force)



import os
import shutil
import tarfile
from typing import Any, Dict, List, Tuple

from sacrerouge.common.util import download_file_from_google_drive, download_url_to_file
from sacrerouge.data import Metrics, MetricsDict
from sacrerouge.datasets.fabbri2020.pair_data import run_pair_data
from sacrerouge.io import JsonlReader, JsonlWriter


def download_human_judgments(output_dir: str, force: bool) -> str:
    url = 'https://storage.googleapis.com/sfr-summarization-repo-research/model_annotations.aligned.jsonl'
    file_path = f'{output_dir}/model_annotations.aligned.jsonl'
    download_url_to_file(url, file_path, force)
    return file_path


def download_system_outputs(output_dir: str, force: bool) -> str:
    expanded_dir = f'{output_dir}/expanded/'
    for system_id in range(0, 24):
        url = f'https://storage.googleapis.com/sfr-summarization-repo-research/M{system_id}.tar.gz'
        file_path = f'{output_dir}/M{system_id}.tar.gz'
        download_url_to_file(url, file_path, force)

        model_expanded_dir = f'{expanded_dir}/M{system_id}'
        if os.path.exists(model_expanded_dir) and force:
            shutil.rmtree(model_expanded_dir)

        if not os.path.exists(model_expanded_dir):
            with tarfile.open(file_path, 'r') as tar:
                tar.extractall(expanded_dir)
    return expanded_dir


def load_judgments(file_path: str) -> Tuple[List[Dict[str, Any]], List[Dict[str, Any]], List[Metrics]]:
    summaries = []
    summaries_with_crowd = []
    metrics_list = []

    with JsonlReader(file_path) as f:
        for instance in f:
            instance_id = instance['id']
            summarizer_id = instance['model_id']
            filename = instance['filepath']
            summary = {'text': instance['decoded']}
            references = instance['references']
            expert_annotations = instance['expert_annotations']
            turker_annotations = instance['turker_annotations']
            document = instance['text']

            # It appears that the first reference is always the ground-truth, the others are crowdsourced, although
            # this is not 100% confirmed. It is only based on me looking through a handful of examples.
            assert len(references) == 11
            references[0] = {
                'summarizer_id': 'ground-truth',
                'summarizer_type': 'reference',
                'text': references[0]
            }
            for i in range(1, 11):
                references[i] = {
                    'summarizer_id': f'turker-{i}',
                    'summarizer_type': 'reference',
                    'text': references[i]
                }

            summaries.append({
                'instance_id': instance_id,
                'summarizer_id': summarizer_id,
                'summarizer_type': 'peer',
                'file_path': filename,
                'document': {'text': document},
                'summary': summary,
                'references': [references[0]]
            })
            summaries_with_crowd.append({
                'instance_id': instance_id,
                'summarizer_id': summarizer_id,
                'summarizer_type': 'peer',
                'file_path': filename,
                'document': {'text': document},
                'summary': summary,
                'references': references
            })

            expert_metrics = MetricsDict({
                'coherence': [annotation['coherence'] for annotation in expert_annotations],
                'consistency': [annotation['consistency'] for annotation in expert_annotations],
                'fluency': [annotation['fluency'] for annotation in expert_annotations],
                'relevance': [annotation['relevance'] for annotation in expert_annotations]
            })
            turker_metrics = MetricsDict({
                'coherence': [annotation['coherence'] for annotation in turker_annotations],
                'consistency': [annotation['consistency'] for annotation in turker_annotations],
                'fluency': [annotation['fluency'] for annotation in turker_annotations],
                'relevance': [annotation['relevance'] for annotation in turker_annotations]
            })
            both = MetricsDict({
                'expert': expert_metrics,
                'turker': turker_metrics
            })
            metrics = Metrics(instance_id, summarizer_id, 'peer', both)
            metrics_list.append(metrics)

    return summaries, summaries_with_crowd, metrics_list


def setup_documents(cnn_tar: str, dailymail_tar: str, output_dir: str, force: bool) -> None:
    cnn_dir = f'{output_dir}/raw/cnndm/cnn'
    dm_dir = f'{output_dir}/raw/cnndm/dailymail'
    for tar_path, target_path in [(cnn_tar, cnn_dir), (dailymail_tar, dm_dir)]:
        if os.path.exists(target_path) and force:
            print(f'Removing {target_path}')
            shutil.rmtree(target_path)

        if not os.path.exists(target_path):
            print(f'Untarring {tar_path} (it\'s pretty slow...)')
            with tarfile.open(tar_path, 'r') as tar:
                tar.extractall(f'{output_dir}/raw/cnndm')


def save_data(data: List[Any], file_path: str) -> None:
    with JsonlWriter(file_path) as out:
        for item in data:
            out.write(item)


def setup(cnn_tar: str, dailymail_tar: str, output_dir: str, force: bool) -> None:
    # Download the expert and turker annotations
    download_human_judgments(f'{output_dir}/raw', force)

    # Download the system outputs
    model_output_dir = download_system_outputs(f'{output_dir}/raw/model-outputs', force)

    # Untar all of the documents
    setup_documents(cnn_tar, dailymail_tar, output_dir, force)

    # Pair together the summaries and input documents. The script will output the
    # results into "model_annotations.aligned.paired.jsonl"
    run_pair_data(data_annotations=f'{output_dir}/raw/model_annotations.aligned.jsonl',
                  model_outputs=model_output_dir,
                  story_files=f'{output_dir}/raw')

    summaries, summaries_with_crowd, metrics = load_judgments(f'{output_dir}/raw/model_annotations.aligned.paired.jsonl')
    save_data(summaries, f'{output_dir}/summaries.jsonl')
    save_data(summaries_with_crowd, f'{output_dir}/summaries-with-crowd.jsonl')
    save_data(metrics, f'{output_dir}/metrics.jsonl')

    # TODO Save all of the documents + summaries that were not judged. I'm not really sure
    # what to do with them because several of the models have different output files. We need
    # to come up with some way of distinguishing them. For instance, M5 has
    # "outputs_rouge.aligned.jsonl" and "outputs_rouge+coh.aligned.jsonl", but the "summaries.jsonl"
    # file will just mark whichever summary was judged as "M5"


import os
import pytest

from sacrerouge.common.testing.metric_test_cases import ReferenceBasedMetricTestCase
from sacrerouge.common.testing.util import sacrerouge_command_exists
from sacrerouge.data import MetricsDict, Pyramid, PyramidAnnotation
from sacrerouge.io import JsonlReader
from sacrerouge.metrics import PyramidScore

_pyramid_file_path = 'datasets/duc-tac/tac2008/v1.0/task1.A.pyramids.jsonl'
_annotation_file_path = 'datasets/duc-tac/tac2008/v1.0/task1.A.pyramid-annotations.jsonl'


class TestPyramidScore(ReferenceBasedMetricTestCase):
    @pytest.mark.skipif(not os.path.exists(_pyramid_file_path), reason='TAC 2008 pyramids file does not exist')
    @pytest.mark.skipif(not os.path.exists(_annotation_file_path), reason='TAC 2008 pyramid annotations file does not exist')
    def test_pyramid_score(self):
        # This is a regression test, not necessarily a test for correctness
        pyramids = {pyramid.instance_id: pyramid for pyramid in JsonlReader(_pyramid_file_path, Pyramid).read()}
        annotations = JsonlReader(_annotation_file_path, PyramidAnnotation).read()
        annotation_pyramids = [pyramids[annotation.instance_id] for annotation in annotations]

        metric = PyramidScore()
        actual_output = metric.score_all(annotations, annotation_pyramids)[:5]
        expected_output = [
            {'modified_pyramid_score': 0.2413793103448276},
            {'modified_pyramid_score': 0.0},
            {'modified_pyramid_score': 0.06896551724137931},
            {'modified_pyramid_score': 0.034482758620689655},
            {'modified_pyramid_score': 0.1724137931034483}
        ]
        for i, (expected, actual) in enumerate(zip(expected_output, actual_output)):
            assert actual.approx_equal(MetricsDict(expected), abs=1e-4), f'Instance {i} not equal. Expected {expected}, actual {actual}'

    def test_command_exists(self):
        assert sacrerouge_command_exists(['pyramid-score'])


import json
import os
import tarfile
import urllib.request
from collections import defaultdict

from sacrerouge.io import JsonlReader, JsonlWriter


def download_raw_data(output_dir: str) -> None:
    urls = [
        'https://worksheets.codalab.org/rest/bundles/0x8dabd302e6674c65b0098a2a258fffb2/contents/blob/',
        'https://worksheets.codalab.org/rest/bundles/0x6c53667688d74786aaba63ba3bcc8e0e/contents/blob/',
        'https://worksheets.codalab.org/rest/bundles/0x57d766e803fe4e00882d58fabf9e6101/contents/blob/'
    ]
    paths = [
        f'{output_dir}/raw/articles.jsonl',
        f'{output_dir}/raw/data.tar.gz',
        f'{output_dir}/raw/lqual.jsonl'  # The output from running their "process.py lqual" script, not the file in the "data" folder
    ]

    os.makedirs(f'{output_dir}/raw', exist_ok=True)
    for url, path in zip(urls, paths):
        if os.path.exists(path):
            print(f'Skipping downloading {path}')
        else:
            urllib.request.urlretrieve(url, path)


def setup_documents(output_dir: str):
    with JsonlReader(f'{output_dir}/raw/articles.jsonl') as f:
        with JsonlWriter(f'{output_dir}/documents.jsonl') as out:
            for instance in f:
                instance_id = instance['id']
                text = instance['text']
                out.write({
                    'instance_id': instance_id,
                    'documents': [
                        {'text': text}
                    ]
                })


def load_metrics(output_dir: str):
    metrics_dicts = defaultdict(dict)
    with JsonlReader(f'{output_dir}/raw/lqual.jsonl') as f:
        for instance in f:
            instance_id = instance['id']
            summarizer_id = instance['system']

            # The 'human' field is the list of individual annotations, the
            # 'gold' field is their average, which we don't need
            hter = instance['prompts']['hter']['human']
            overall = instance['prompts']['overall']['human']
            grammar = instance['prompts']['grammar']['human']
            redundancy = instance['prompts']['redundancy']['human']

            # The automatic metrics are identical for all prompts
            metrics_dicts[instance_id][summarizer_id] = {
                'hter': hter,
                'overall': overall,
                'grammar': grammar,
                'redundancy': redundancy,
                'bleu-2': instance['prompts']['hter']['bleu-2'],
                'bleu-4': instance['prompts']['hter']['bleu-4'],
                'meteor': instance['prompts']['hter']['meteor'],
                'ter': instance['prompts']['hter']['ter'],
                'sim': instance['prompts']['hter']['sim'],
                'rouge-1': {'recall': instance['prompts']['hter']['ROUGE-1']},
                'rouge-2': {'recall': instance['prompts']['hter']['ROUGE-2']},
                'rouge-3': {'recall': instance['prompts']['hter']['ROUGE-3']},
                'rouge-4': {'recall': instance['prompts']['hter']['ROUGE-4']},
                'rouge-l': {'recall': instance['prompts']['hter']['ROUGE-L']},
                'rouge-su4': {'recall': instance['prompts']['hter']['ROUGE-SU4']}
            }
        return metrics_dicts


def load_summaries(output_dir: str):
    summaries = defaultdict(dict)
    with tarfile.open(f'{output_dir}/raw/data.tar.gz', 'r') as tar:
        lines = tar.extractfile('./lqual.jsonl').read().decode().splitlines()
        for line in lines:
            data = json.loads(line)
            instance_id = data['input']['contents']['id']
            text = data['input']['contents']['text']

            for summarizer_id in data['input']['contents']['system'].split(';'):
                if summarizer_id == 'reference':
                    summarizer_type = 'reference'
                else:
                    summarizer_type = 'peer'

                summary = {
                    'summarizer_id': summarizer_id,
                    'summarizer_type': summarizer_type,
                    'text': text
                }
                summaries[instance_id][summarizer_id] = summary
    return summaries


def save_data(summaries, metrics_dicts, output_dir: str):
    with JsonlWriter(f'{output_dir}/summaries.jsonl') as out_summaries:
        with JsonlWriter(f'{output_dir}/metrics.jsonl') as out_metrics:
            for instance_id in sorted(metrics_dicts.keys()):
                for summarizer_id in sorted(metrics_dicts[instance_id].keys()):
                    summary = summaries[instance_id][summarizer_id]
                    metrics = metrics_dicts[instance_id][summarizer_id]

                    if summary['summarizer_type'] == 'reference':
                        references = []
                    else:
                        references = [summaries[instance_id]['reference']]

                    out_summaries.write({
                        'instance_id': instance_id,
                        'summarizer_id': summarizer_id,
                        'summarizer_type': summary['summarizer_type'],
                        'summary': summary,
                        'references': references
                    })
                    out_metrics.write({
                        'instance_id': instance_id,
                        'summarizer_id': summarizer_id,
                        'summarizer_type': summary['summarizer_type'],
                        'metrics': {
                            'chaganty2018': metrics
                        }
                    })


def setup(output_dir: str):
    os.makedirs(output_dir, exist_ok=True)
    download_raw_data(output_dir)
    setup_documents(output_dir)

    metrics_dicts = load_metrics(output_dir)
    summaries = load_summaries(output_dir)
    save_data(summaries, metrics_dicts, output_dir)



import argparse
import lxml.html
import re
import tarfile
from collections import defaultdict
from io import BytesIO
from lxml import etree
from nltk.tokenize import sent_tokenize

from sacrerouge.io import JsonlWriter


def parse_story(root):
    headline = None
    headline_nodes = list(root.xpath('body/doc/headline'))
    if len(headline_nodes) > 0:
        headline = headline_nodes[0].text_content().strip()

    text = []
    for p_node in root.xpath('body/doc/text/p'):
        paragraph = p_node.text_content().strip()
        paragraph = re.sub('\s+', ' ', paragraph)
        paragraph = sent_tokenize(paragraph)
        text.append(paragraph)

    return headline, text


def parse_other(root):
    # Everything is combined into a single paragraph.
    text = root.xpath('//text')[0].text_content().strip()
    text = re.sub('\s+', ' ', text)
    sentences = [sent_tokenize(text)]
    return sentences


def load_documents(documents_tar: str):
    selectors = {}
    documents = defaultdict(lambda: defaultdict(list))

    with tarfile.open(documents_tar, 'r') as tar:
        inner_tar_bytes = tar.extractfile('TAC2009_Summarization_Documents/UpdateSumm09_test_docs_files.tar.gz').read()
        with tarfile.open(fileobj=BytesIO(inner_tar_bytes)) as inner_tar:
            for member in inner_tar.getmembers():
                if member.isfile():
                    path = member.name.split('/')
                    instance_id = path[1][:-1].lower()
                    selector = path[1][-1]
                    group = path[2][-1]
                    filename = path[-1]

                    if instance_id not in selectors:
                        selectors[instance_id] = selector

                    date = filename[8:16]
                    year, month, day = date[:4], date[4:6], date[6:]
                    date = f'{year}-{month}-{day}'

                    html = inner_tar.extractfile(member).read().decode()
                    tree = lxml.html.document_fromstring(html)

                    # The documents come from English Gigaword. Different document
                    # types have different structures, some of which we can parse
                    # ("story"), but others we cannot ("other")
                    document_type = tree.xpath('body/doc')[0].get('type')
                    if document_type == 'story':
                        headline, text = parse_story(tree)
                    else:
                        headline = None
                        text = parse_other(tree)

                    document = {
                        'filename': filename,
                        'document_type': document_type,
                        'headline': headline,
                        'date': date,
                        'text': text
                    }
                    documents[instance_id][group].append(document)

    return documents, selectors


def load_summaries(eval_tar: str) -> None:
    summaries = defaultdict(lambda: defaultdict(list))
    with tarfile.open(eval_tar, 'r') as tar:
        for member in tar.getmembers():
            if member.isfile() and member.name.startswith('UpdateSumm09_eval/ROUGE/models/'):
                path = member.name.split('/')
                filename = path[-1]
                parts = filename.split('.')
                instance_id = parts[0].split('-')[0].lower()
                group = parts[0].split('-')[1]
                annotator = parts[4]

                text = tar.extractfile(member).read().decode().strip()
                sentences = sent_tokenize(text)
                summary = {
                    'annotator': annotator,
                    'text': sentences
                }
                summaries[instance_id][group].append(summary)
    return summaries


def load_topics(file_path: str):
    topics = {}
    tree = etree.parse(file_path)
    for topic_node in tree.xpath('//topic'):
        instance_id = topic_node.get('id')[:-1].lower()
        title = topic_node.xpath('title')[0].text.strip()
        narrative = sent_tokenize(topic_node.xpath('narrative')[0].text.strip())
        topics[instance_id] = {
            'title': title,
            'narrative': narrative
        }
    return topics


def save_instances(documents, selectors, summaries, topics, output_dir: str) -> None:
    with JsonlWriter(f'{output_dir}/task1.A-B.jsonl') as out_A_B:
        with JsonlWriter(f'{output_dir}/task1.A.jsonl') as out_A:
            with JsonlWriter(f'{output_dir}/task1.B.jsonl') as out_B:
                for instance_id in sorted(documents.keys()):
                    topic = topics[instance_id]
                    selector = selectors[instance_id]

                    documents_A = documents[instance_id]['A']
                    documents_B = documents[instance_id]['B']

                    summaries_A = summaries[instance_id]['A']
                    summaries_B = summaries[instance_id]['B']

                    out_A.write({
                        'instance_id': f'{instance_id}-A',
                        'selector': selector,
                        'topic': topic,
                        'documents': documents_A,
                        'summaries': summaries_A
                    })
                    out_B.write({
                        'instance_id': f'{instance_id}-B',
                        'selector': selector,
                        'topic': topic,
                        'documents': documents_B,
                        'summaries': summaries_B
                    })
                    out_A_B.write({
                        'instance_id': instance_id,
                        'selector': selector,
                        'topic': topic,
                        'documents_A': documents_A,
                        'documents_B': documents_B,
                        'summaries_A': summaries_A,
                        'summaries_B': summaries_B
                    })


def setup(data_dir: str, output_dir: str):
    documents_tar = f'{data_dir}/from-nist/TAC2009_Summarization_Documents.tgz'
    eval_tar = f'{data_dir}/scrapes/tac.nist.gov/protected/past-aquaint2/2009/UpdateSumm09_eval.tar.gz'
    topics_file = f'{data_dir}/scrapes/tac.nist.gov/protected/past-aquaint2/2009/UpdateSumm09_test_topics.xml.txt'
    main(documents_tar, eval_tar, topics_file, output_dir)


def main(documents_tar, eval_tar, topics_file, output_dir):
    documents, selectors = load_documents(documents_tar)
    summaries = load_summaries(eval_tar)
    topics = load_topics(topics_file)
    save_instances(documents, selectors, summaries, topics, output_dir)


if __name__ == '__main__':
    argp = argparse.ArgumentParser()
    argp.add_argument('documents_tar')
    argp.add_argument('eval_tar')
    argp.add_argument('topics_file')
    argp.add_argument('output_dir')
    args = argp.parse_args()

    main(args.documents_tar, args.eval_tar, args.topics_file, args.output_dir)



import argparse
from overrides import overrides

from sacrerouge.datasets.multiling.multiling2015 import mds, sds, sds_metrics
from sacrerouge.commands import DatasetSetupSubcommand
from sacrerouge.common.util import download_file_from_google_drive


@DatasetSetupSubcommand.register('multiling2015')
class MultiLing2015Subcommand(DatasetSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the MultiLing 2015 dataset'
        self.parser = parser.add_parser('multiling2015', description=description, help=description)
        self.parser.add_argument(
            'output_dir',
            type=str,
            help='The directory where the data should be saved'
        )
        self.parser.add_argument(
            '--train-zip',
            type=str,
            help='The path to the "TrainingData2015-20150314.zip" file'
        )
        self.parser.add_argument(
            '--test-zip',
            type=str,
            help='The path to the "SourceTextsV2b-20150214.zip" file'
        )
        self.parser.set_defaults(subfunc=self.run)

    def _notify_about_license(self):
        print('This dataset is distributed by MultiLing 2015. If you use this dataset, please refer to their '
              'website (http://multiling.iit.demokritos.gr/pages/view/1516/multiling-2015) '
              'for instructions on proper attribution.')
        print()
        response = input('Continue? [y/n]: ')
        if response.lower() != 'y':
            exit()

    @overrides
    def run(self, args):
        self._notify_about_license()

        # Download the dataset so each individual setup does not need
        # to repeat this. The dataset comes from
        # http://multiling.iit.demokritos.gr/pages/view/1532/task-mss-single-document-summarization-data-and-information
        tar_path = f'{args.output_dir}/multilingMss2015Eval.tar.gz'
        download_file_from_google_drive('0B31rakzMfTMZa0ZIcmgzREstcVE', tar_path)
        sds.setup(tar_path, f'{args.output_dir}/sds')

        # The SDS metrics file was provided by John Conroy
        eval_tar_path = f'{args.output_dir}/Evaluation_MultiLing2015_MSS.tgz'
        download_file_from_google_drive('1j_jV9JAc0t_EulCUMH7Y-dQSpqD9BpFd', eval_tar_path)
        sds_metrics.setup(eval_tar_path, f'{args.output_dir}/sds')

        # The MDS data is password protected, so the user must provide the zips
        if not all([args.train_zip, args.test_zip]):
            print('Skipping setting up MDS data because either the training or testing zip is missing')
        else:
            mds.setup(args.train_zip, args.test_zip, f'{args.output_dir}/mds')


import unittest

from sacrerouge.common.testing.util import sacrerouge_command_exists


class TestMultiLing2017Subcommand(unittest.TestCase):
    def test_command_exists(self):
        assert sacrerouge_command_exists(['setup-dataset', 'multiling2017'])



import logging
from typing import List

from sacrerouge.data import EvalInstance, Pyramid, PyramidAnnotation
from sacrerouge.data.dataset_readers import DatasetReader
from sacrerouge.data.fields import Fields, PyramidField, PyramidAnnotationField, ReferencesField, SummaryField
from sacrerouge.io import JsonlReader

logger = logging.getLogger(__name__)


@DatasetReader.register('pyramid-based')
class PyramidBasedDatasetReader(DatasetReader):
    def __init__(self, include_reference_annotations: bool = True) -> None:
        """
        Args:
            include_reference_annotations: Indicates if pyramid annotations should be created
                for all of the summaries in the pyramid. This will be done for all of the instance_ids
                which are observed in the annotations file.
        """
        super().__init__()
        self.include_reference_annotations = include_reference_annotations

    def read(self,
             pyramid_jsonl: str,
             annotation_jsonl: str) -> List[EvalInstance]:
        logger.info(f'Loading Pyramids from {pyramid_jsonl}')
        pyramids = {}
        with JsonlReader(pyramid_jsonl, Pyramid) as f:
            for pyramid in f:
                pyramids[pyramid.instance_id] = pyramid
        logger.info(f'Loaded {len(pyramids)} pyramids')

        logger.info(f'Loading Pyramid annotations from {annotation_jsonl}')
        instances = []
        instance_ids = set()
        with JsonlReader(annotation_jsonl, PyramidAnnotation) as f:
            for annotation in f:
                fields = Fields({
                    'annotation': PyramidAnnotationField(annotation),
                    'pyramid': PyramidField(pyramids[annotation.instance_id])
                })

                instance = EvalInstance(
                    annotation.instance_id,
                    annotation.summarizer_id,
                    annotation.summarizer_type,
                    fields
                )
                instances.append(instance)

                instance_ids.add(annotation.instance_id)

            logger.info(f'Loaded {len(instances)} Pyramid annotations')

        if self.include_reference_annotations:
            logger.info(f'Generating Pyramid annotations for the reference summaries')
            reference_instances = []
            for instance_id in instance_ids:
                pyramid = pyramids[instance_id]

                # We can only do this if there are > 1 summaries used to construct the pyramid
                if len(pyramid.summarizer_ids) > 1:
                    for i in range(len(pyramid.summarizer_ids)):
                        annotation = pyramid.get_annotation(i)
                        reduced_pyramid = pyramid.remove_summary(i)
                        fields = Fields({
                            'annotation': PyramidAnnotationField(annotation),
                            'pyramid': PyramidField(reduced_pyramid)
                        })
                        instance = EvalInstance(
                            annotation.instance_id,
                            annotation.summarizer_id,
                            annotation.summarizer_type,
                            fields
                        )
                        reference_instances.append(instance)
            logger.info(f'Generated {len(reference_instances)} reference summary annotations')
            instances.extend(reference_instances)

        logger.info(f'Loaded a total of {len(instances)} instances')
        return instances



import os
import zipfile
from collections import defaultdict
from io import BytesIO
from nltk import sent_tokenize
from typing import Any, Dict

from sacrerouge.data import Metrics, MetricsDict
from sacrerouge.io import JsonlWriter
from sacrerouge.datasets.multiling.util import LANGUAGE_CODES


FILE_ID_TO_INSTANCE_ID = {
    1: 'M002',
    2: 'M003',
    3: 'M001',
    4: 'M000',
    5: 'M007',
    6: 'M005',
    7: 'M004',
    8: 'M006',
    9: 'M008',
    10: 'M009'
}


def load_reference_summaries(data_path: str,
                             summaries: Dict[str, Dict[str, Dict[str, Any]]]):
    with zipfile.ZipFile(data_path, 'r') as zip:
        summaries_zip_bytes = zip.read('PublishedCorpusFileSet/MultiLingPilot2011-ModelSummaries.zip')
        with zipfile.ZipFile(BytesIO(summaries_zip_bytes)) as summaries_zip:
            for name in summaries_zip.namelist():
                if name.endswith('.250'):
                    path = name.split('/')
                    language = path[0].lower()
                    filename = path[1]
                    parts = filename.split('.')
                    instance_id = parts[0][:4]
                    summarizer_id = parts[1]

                    # The summaries don't seem to have any structure, so
                    # just read in the full text.
                    lines = summaries_zip.read(name).decode().splitlines()
                    clean_lines = []
                    for line in lines:
                        line = line.strip()
                        if line:
                            clean_lines.append(line)
                    text = ' '.join(clean_lines)
                    # Sentence tokenize English only
                    if language == 'english':
                        text = sent_tokenize(text)

                    summary = {
                        'summarizer_id': summarizer_id,
                        'summarizer_type': 'reference',
                        'text': text
                    }
                    summaries[language][instance_id][summarizer_id] = summary


def load_peer_summaries(data_path: str,
                        summaries: Dict[str, Dict[str, Dict[str, Any]]]):
    with zipfile.ZipFile(data_path, 'r') as zip:
        summaries_zip_bytes = zip.read('PublishedCorpusFileSet/MultiLingPilot2011-PeerSummaries.zip')
        with zipfile.ZipFile(BytesIO(summaries_zip_bytes)) as summaries_zip:
            for name in summaries_zip.namelist():
                filename = os.path.basename(name)
                if filename.startswith('M00'):
                    path = name.split('/')
                    summarizer_id = path[1][2:]
                    language = path[2].lower()
                    instance_id = path[3][:4]

                    # Lines are roughly sentences (not always), so we keep them intact
                    lines = summaries_zip.read(name).decode().splitlines()
                    clean_lines = []
                    for line in lines:
                        line = line.strip()
                        if line:
                            clean_lines.append(line)
                    lines = clean_lines

                    summary = {
                        'summarizer_id': summarizer_id,
                        'summarizer_type': 'peer',
                        'text': lines
                    }
                    summaries[language][instance_id][summarizer_id] = summary


def load_autosummeng_output(data_path: str, metrics):
    with zipfile.ZipFile(data_path, 'r') as zip:
        eval_zip_bytes = zip.read('PublishedCorpusFileSet/MultiLingPilot2011-EvaluationResults.zip')
        with zipfile.ZipFile(BytesIO(eval_zip_bytes)) as eval_zip:
            for name in eval_zip.namelist():
                if name.startswith('PublishedEvaluation/AutomaticEvaluation/AutoSummENG/Detailed/') and name.endswith('.csv'):
                    filename = os.path.basename(name)
                    # strip "Detailed.csv" off the end to get the language name
                    language = filename[:-12]

                    lines = eval_zip.read(name).decode().splitlines()
                    for line in lines[1:]:
                        columns = line.split()
                        instance_id = columns[0]
                        summarizer_id = columns[1][2:]
                        # There are only scores for peers, so no jackknifing
                        metrics[language][instance_id][summarizer_id]['MeMoG'] = float(columns[3])


def load_rouge_output(data_path: str, metrics):
    with zipfile.ZipFile(data_path, 'r') as zip:
        eval_zip_bytes = zip.read('PublishedCorpusFileSet/MultiLingPilot2011-EvaluationResults.zip')
        with zipfile.ZipFile(BytesIO(eval_zip_bytes)) as eval_zip:
            for name in eval_zip.namelist():
                if name.startswith('PublishedEvaluation/AutomaticEvaluation/ROUGE/Detailed/') and name.endswith('.txt'):
                    filename = os.path.basename(name)
                    # strip "Detailed.txt" off the end to get the language name
                    language = filename[:-12]

                    lines = eval_zip.read(name).decode().splitlines()
                    for line in lines:
                        columns = line.split()
                        if len(columns) == 7 and columns[2] == 'Eval':
                            summarizer_id = columns[0]
                            rouge_metric = columns[1].lower()
                            # The file_id and summarizer_id appear like "5.ID3". The file_id
                            # seems to have no correspondence with the instance_id. We found
                            # this mapping by computing the rouge score ourselves and then
                            # finding the mapping
                            file_id = int(columns[3].split('.')[0])
                            instance_id = FILE_ID_TO_INSTANCE_ID[file_id]
                            summarizer_id = columns[3].split('.')[1][2:]

                            recall = float(columns[4][2:]) * 100
                            precision = float(columns[5][2:]) * 100
                            f1 = float(columns[6][2:]) * 100
                            # Only peers evaluated, no jackknifing
                            metrics[language][instance_id][summarizer_id][rouge_metric] = {
                                'recall': recall,
                                'precision': precision,
                                'f1': f1
                            }


def load_manual_output(data_path: str, metrics):
    with zipfile.ZipFile(data_path, 'r') as zip:
        eval_zip_bytes = zip.read('PublishedCorpusFileSet/MultiLingPilot2011-EvaluationResults.zip')
        with zipfile.ZipFile(BytesIO(eval_zip_bytes)) as eval_zip:
            for name in eval_zip.namelist():
                if name.startswith('PublishedEvaluation/HumanEvaluation/') and name.endswith('.csv'):
                    filename = os.path.basename(name)
                    # strip "Grades.csv" off the end to get the language name
                    language = filename[:-10]

                    lines = eval_zip.read(name).decode().splitlines()
                    for line in lines[1:]:
                        columns = line.split()
                        # Strip the quotes from the instance and summarizer
                        instance_id = columns[0][1:-1]
                        summarizer_id = columns[1][3:-1]

                        if 'grade' not in metrics[language][instance_id][summarizer_id]:
                            metrics[language][instance_id][summarizer_id]['grade'] = []
                        if 'length_aware_grade' not in metrics[language][instance_id][summarizer_id]:
                            metrics[language][instance_id][summarizer_id]['length_aware_grade'] = []

                        metrics[language][instance_id][summarizer_id]['grade'].append(int(columns[4]))
                        metrics[language][instance_id][summarizer_id]['length_aware_grade'].append(float(columns[6]))


def get_references(summaries, instance_id, summarizer_id):
    summarizer_ids = list(summaries[instance_id].keys())
    reference_ids = list(filter(lambda sid: sid.isalpha(), summarizer_ids))

    references = []
    for reference_id in reference_ids:
        if summarizer_id == reference_id:
            continue
        references.append(summaries[instance_id][reference_id])
    return references


def save_data(summaries, metrics, output_dir: str):
    for language in summaries.keys():
        code = LANGUAGE_CODES[language]

        with JsonlWriter(f'{output_dir}/{code}.summaries.jsonl') as out_summaries:
            with JsonlWriter(f'{output_dir}/{code}.metrics.jsonl') as out_metrics:
                for instance_id in sorted(summaries[language].keys()):
                    for summarizer_id in sorted(summaries[language][instance_id].keys()):
                        summary = summaries[language][instance_id][summarizer_id]
                        instance_metrics = metrics[language][instance_id][summarizer_id]
                        references = get_references(summaries[language], instance_id, summarizer_id)

                        out_summaries.write({
                            'instance_id': instance_id,
                            'summarizer_id': summarizer_id,
                            'summarizer_type': summary['summarizer_type'],
                            'summary': summary,
                            'references': references
                        })
                        out_metrics.write(Metrics(instance_id, summarizer_id, summary['summarizer_type'], instance_metrics))


def setup(data_path: str, output_dir: str):
    summaries = defaultdict(lambda: defaultdict(dict))
    load_reference_summaries(data_path, summaries)
    load_peer_summaries(data_path, summaries)

    metrics = defaultdict(lambda: defaultdict(MetricsDict))
    load_autosummeng_output(data_path, metrics)
    load_rouge_output(data_path, metrics)
    load_manual_output(data_path, metrics)
    save_data(summaries, metrics, output_dir)



import os
import pytest
import unittest
from collections import defaultdict
from typing import Dict, List

from sacrerouge.data import Metrics
from sacrerouge.io import JsonlReader

_task1_metrics_file_path = 'datasets/duc-tac/duc2003/v1.0/task1.metrics.jsonl'
_task2_metrics_file_path = 'datasets/duc-tac/duc2003/v1.0/task2.metrics.jsonl'
_task3_metrics_file_path = 'datasets/duc-tac/duc2003/v1.0/task3.metrics.jsonl'
_task4_metrics_file_path = 'datasets/duc-tac/duc2003/v1.0/task4.metrics.jsonl'


class TestDUC2003Metrics(unittest.TestCase):
    def _convert_to_dicts(self, metrics_list: List[Metrics]) -> Dict[str, Dict[str, Metrics]]:
        metrics_dicts = defaultdict(dict)
        for metrics in metrics_list:
            metrics_dicts[metrics.instance_id][metrics.summarizer_id] = metrics.metrics
        return metrics_dicts

    @pytest.mark.skipif(not os.path.exists(_task1_metrics_file_path), reason='DUC 2003 task 1 metrics file does not exist')
    def test_task1_system_level(self):
        summary_level_metrics = JsonlReader(_task1_metrics_file_path, Metrics).read()
        summary_level_metrics = self._convert_to_dicts(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files
        # results/original.results.corrected.improved.withPenaltyForExcess/results.table1
        assert summary_level_metrics['NYT19990925.0263']['10']['length_adjusted_coverage']['median'] == 0.267
        assert summary_level_metrics['NYT19981204.0331']['17']['coverage']['mean'] == 0.2
        assert summary_level_metrics['XIE19980211.0079']['18']['coverage']['median'] == 0.0

        # results/results.table.U.paired
        assert summary_level_metrics['NYT19990925.0263']['7']['usefulness'] == [2, 1]
        assert summary_level_metrics['APW19981005.0718']['H']['usefulness'] == [3, 2]
        assert summary_level_metrics['APW19981123.0532']['25']['usefulness'] == [1, 1]

    @pytest.mark.skipif(not os.path.exists(_task2_metrics_file_path), reason='DUC 2003 task 2 metrics file does not exist')
    def test_task2_system_level(self):
        summary_level_metrics = JsonlReader(_task2_metrics_file_path, Metrics).read()
        summary_level_metrics = self._convert_to_dicts(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files
        # results/original.results.corrected.improved.withPenaltyForExcess/results.table2
        assert summary_level_metrics['d30003']['10']['peer_quality']['Q1'] == 1
        assert summary_level_metrics['d30034']['13']['coverage']['std'] == 0.316
        assert summary_level_metrics['d31013']['15']['peer_quality']['Q2'] == 2

    @pytest.mark.skipif(not os.path.exists(_task3_metrics_file_path), reason='DUC 2003 task 3 metrics file does not exist')
    def test_task3_system_level(self):
        summary_level_metrics = JsonlReader(_task3_metrics_file_path, Metrics).read()
        summary_level_metrics = self._convert_to_dicts(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files
        # results/original.results.corrected.improved.withPenaltyForExcess/results.table3
        assert summary_level_metrics['d111']['17']['peer_quality']['Q4'] == 1
        assert summary_level_metrics['d123']['16']['unmarked_units_related_to_subject'] == 1.0
        assert summary_level_metrics['d125']['B']['num_peer_units'] == 6

    @pytest.mark.skipif(not os.path.exists(_task4_metrics_file_path), reason='DUC 2003 task 4 metrics file does not exist')
    def test_task4_system_level(self):
        summary_level_metrics = JsonlReader(_task4_metrics_file_path, Metrics).read()
        summary_level_metrics = self._convert_to_dicts(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files
        # results/original.results.corrected.improved.withPenaltyForExcess/results.table4
        assert summary_level_metrics['d305']['10']['length_adjusted_coverage']['std'] == 0.079
        assert summary_level_metrics['d365']['14']['unmarked_units_related_to_subject'] == 1.0
        assert summary_level_metrics['d382']['C']['num_peer_units'] == 6

        # results/results.table.R.paired
        assert summary_level_metrics['d305']['10']['responsiveness'] == [4, 2]
        assert summary_level_metrics['d326']['E']['responsiveness'] == [4, 3]
        assert summary_level_metrics['d432']['19']['responsiveness'] == [1, 1]



import itertools
import os
import pytest
import unittest

from sacrerouge.common import DATA_ROOT
from sacrerouge.common.testing import FIXTURES_ROOT
from sacrerouge.io import JsonlReader
from sacrerouge.metrics import SIMetrix

_simetrix_jar = f'{DATA_ROOT}/metrics/simetrix/simetrix.jar'
_instances_file_path = f'{FIXTURES_ROOT}/data/simetrix/instances.jsonl'
_summary_metrics_file_path = f'{FIXTURES_ROOT}/data/simetrix/metrics.summary-level.jsonl'
_system_metrics_file_path = f'{FIXTURES_ROOT}/data/simetrix/metrics.system-level.jsonl'


class TestSIMetrix(unittest.TestCase):
    def _setup_multi_all(self, instances):
        instance_id_to_index = {}
        summarizer_id_to_index = {}
        summaries_lists = []
        documents_lists = []

        instances = instances[:]
        instances.sort(key=lambda instance: instance['instance_id'])
        for i, (instance_id, group) in enumerate(itertools.groupby(instances, key=lambda instance: instance['instance_id'])):
            instance_id_to_index[instance_id] = i
            group = list(group)
            group.sort(key=lambda instance: instance['summarizer_id'])

            # Take the documents from the first instance
            first = group[0]
            documents_lists.append(first['documents'])

            # Add the summaries
            summaries_list = []
            for j, instance in enumerate(group):
                summarizer_id = instance['summarizer_id']
                summarizer_id_to_index[summarizer_id] = j
                summaries_list.append(instance['summary'])
            summaries_lists.append(summaries_list)

        return summaries_lists, documents_lists, instance_id_to_index, summarizer_id_to_index

    @pytest.mark.skipif(not os.path.exists(_simetrix_jar), reason='SIMetrix jar does not exist')
    @pytest.mark.skipif(not os.path.exists(_instances_file_path), reason='SIMetrix data does not exist')
    @pytest.mark.skipif(not os.path.exists(_summary_metrics_file_path), reason='SIMetrix data does not exist')
    def test_score_multi_all(self):
        instances = JsonlReader(_instances_file_path).read()
        summaries_lists, documents_lists, instance_id_to_index, summarizer_id_to_index = self._setup_multi_all(instances)

        simetrix = SIMetrix()
        metrics_lists = simetrix.score_multi_all(summaries_lists, documents_lists)
        for expected_metrics in JsonlReader(_summary_metrics_file_path).read():
            instance_index = instance_id_to_index[expected_metrics['instance_id']]
            summarizer_index = summarizer_id_to_index[expected_metrics['summarizer_id']]
            actual_metrics = metrics_lists[instance_index][summarizer_index]
            assert actual_metrics == expected_metrics['metrics']

    @pytest.mark.skipif(not os.path.exists(_simetrix_jar), reason='SIMetrix jar does not exist')
    @pytest.mark.skipif(not os.path.exists(_instances_file_path), reason='SIMetrix data does not exist')
    @pytest.mark.skipif(not os.path.exists(_system_metrics_file_path), reason='SIMetrix data does not exist')
    def test_evaluate(self):
        instances = JsonlReader(_instances_file_path).read()
        simetrix = SIMetrix()

        summarizer_id_to_metrics = {}
        for expected_metrics in JsonlReader(_system_metrics_file_path).read():
            summarizer_id = expected_metrics['summarizer_id']
            summarizer_id_to_metrics[summarizer_id] = expected_metrics

        instances.sort(key=lambda instance: instance['summarizer_id'])
        for summarizer_id, group in itertools.groupby(instances, key=lambda instance: instance['summarizer_id']):
            group = list(group)
            summaries = [member['summary'] for member in group]
            documents_list = [member['documents'] for member in group]
            actual_metrics, _ = simetrix.evaluate(summaries, documents_list)
            expected_metrics = summarizer_id_to_metrics[summarizer_id]['metrics']

            assert len(actual_metrics) == len(expected_metrics)
            for metric, value in actual_metrics.items():
                assert value == expected_metrics['Avg' + metric]



import argparse
import json
import logging
import os
import scipy.stats
from overrides import overrides
from typing import Any, Dict, Tuple

from sacrerouge.commands import RootSubcommand
from sacrerouge.common.logging import prepare_global_logging

logger = logging.getLogger(__name__)


def _get_equations(alternative: str) -> Tuple[str, str]:
    # Returns the equations for the null and alternative hypotheses based on the description in
    # https://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.wilcoxon.html
    if alternative == 'two-sided':
        return 'A = B', 'A != B'
    if alternative == 'greater':
        return 'A < B', 'A > B'
    if alternative == 'less':
        return 'A > B', 'A < B'
    raise Exception(f'Unknown alternative: {alternative}')


def run_wilcoxon_tests(correlations_A: Dict[str, Dict[str, float]],
                       correlations_B: Dict[str, Dict[str, float]],
                       alternative: str = 'two-sided') -> Dict[str, Any]:
    null_eq, alternative_eq = _get_equations(alternative)
    results = {
        'alternative': alternative,
        'null_equation': null_eq,
        'alternative_equation': alternative_eq
    }

    coefficients = set(correlations_A.keys()) & set(correlations_B.keys())
    for coef in coefficients:
        logger.info(f'Running Wilcoxon test on the {coef} summary-level correlations')

        instance_ids = set(correlations_A[coef].keys()) & set(correlations_B[coef].keys())
        logger.info(f'Found {len(instance_ids)} common instance correlations')

        if len(instance_ids) == 0:
            logger.warning('Found no common instance correlations. Skipping coefficient')
            continue

        values_A = []
        values_B = []
        for instance_id in instance_ids:
            values_A.append(correlations_A[coef][instance_id])
            values_B.append(correlations_B[coef][instance_id])

        statistic, p_value = scipy.stats.wilcoxon(values_A, values_B, alternative=alternative)
        results[coef] = {'statistic': statistic, 'p_value': p_value, 'n': len(values_A)}

        logger.info(f'Test statistic: {statistic}, p-value: {p_value}')

    return results


@RootSubcommand.register('stat-sig-test')
class StatisticalSignificanceTestSubcommand(RootSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Run a Wilcoxon signed-rank test on two sets of summary-level correlations'
        self.parser = parser.add_parser('stat-sig-test', description=description, help=description)
        self.parser.add_argument(
            '--summary-level-correlations-A',
            type=str,
            help='The individual summary-level correlations for metric A',
            required=True
        )
        self.parser.add_argument(
            '--summary-level-correlations-B',
            type=str,
            help='The individual summary-level correlations for metric B',
            required=True
        )
        self.parser.add_argument(
            '--output-file',
            type=str,
            help='The json output file which will contain the test results'
        )
        self.parser.add_argument(
            '--log-file',
            type=str,
            help='The file where the log should be written'
        )
        self.parser.add_argument(
            '--silent',
            action='store_true',
            help='Controls whether the log should be written to stdout'
        )
        self.parser.add_argument(
            '--alternative',
            type=str,
            choices=['two-sided', 'greater', 'less'],
            default='two-sided',
            help='The alternative hypothesis for the test. See https://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.wilcoxon.html for details'
        )
        self.parser.set_defaults(func=self.run)

    def run(self, args):
        prepare_global_logging(file_path=args.log_file, silent=args.silent)

        correlations_A = json.load(open(args.summary_level_correlations_A, 'r'))
        correlations_B = json.load(open(args.summary_level_correlations_B, 'r'))

        results = run_wilcoxon_tests(correlations_A, correlations_B, alternative=args.alternative)

        if args.output_file:
            dirname = os.path.dirname(args.output_file)
            if dirname:
                os.makedirs(dirname, exist_ok=True)
            with open(args.output_file, 'w') as out:
                out.write(json.dumps(results, indent=2))

        if not args.silent:
            logger.info(json.dumps(results, indent=2))


import logging
from typing import List

from sacrerouge.data import EvalInstance
from sacrerouge.data.dataset_readers import DatasetReader
from sacrerouge.data.fields import ReferencesField, Fields, SummaryField
from sacrerouge.io import JsonlReader

logger = logging.getLogger(__name__)


@DatasetReader.register('reference-based')
class ReferenceBasedDatasetReader(DatasetReader):
    def read(self, input_jsonl: str) -> List[EvalInstance]:
        logger.info(f'Loading evaluation instances from {input_jsonl}')
        instances = []
        with JsonlReader(input_jsonl) as f:
            for data in f:
                fields = {}
                fields['summary'] = SummaryField(data['summary']['text'])

                if 'reference' in data:
                    fields['references'] = ReferencesField([data['reference']['text']])
                else:
                    fields['references'] = ReferencesField([reference['text'] for reference in data['references']])
                fields = Fields(fields)

                instance = EvalInstance(
                    data['instance_id'],
                    data['summarizer_id'],
                    data['summarizer_type'],
                    fields
                )
                instances.append(instance)

            logger.info(f'Loaded {len(instances)} instances')
            return instances



import argparse

from sacrerouge.common import Registrable
from sacrerouge.common.util import import_module_and_submodules
from sacrerouge.commands import RootSubcommand, metric_command


def build_argument_parser():
    # Ensure all of the subcommands have been loaded
    import_module_and_submodules('sacrerouge')

    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()

    # Add all of the root-level commands using the registry
    for name, (cls_, _) in sorted(Registrable._registry[RootSubcommand].items()):
        cls_().add_subparser(subparsers)

    # Add a command for each individual metric
    metric_command.add_metric_subcommands(subparsers)
    return parser


import argparse
import logging
import os
from collections import defaultdict
from overrides import overrides
from subprocess import Popen, PIPE
from typing import List

from sacrerouge.commands import MetricSetupSubcommand
from sacrerouge.common import DATA_ROOT, TemporaryDirectory
from sacrerouge.common.util import command_exists
from sacrerouge.data import MetricsDict
from sacrerouge.data.jackknifers import ReferencesJackknifer
from sacrerouge.data.types import ReferenceType, SummaryType
from sacrerouge.metrics import Metric, ReferenceBasedMetric

logger = logging.getLogger(__name__)


@Metric.register('autosummeng')
class AutoSummENG(ReferenceBasedMetric):
    def __init__(self,
                 min_n: int = 3,
                 max_n: int = 3,
                 d_window: int = 3,
                 min_score: float = 0.0,
                 max_score: float = 1.0,
                 autosummeng_root: str = f'{DATA_ROOT}/metrics/AutoSummENG',
                 verbose: bool = False):
        super().__init__(['summary'], ['references'], jackknifer=ReferencesJackknifer())
        self.min_n = min_n
        self.max_n = max_n
        self.d_window = d_window
        self.min_score = min_score
        self.max_score = max_score
        self.autosummeng_root = autosummeng_root
        self.verbose = verbose

        if not os.path.exists(autosummeng_root):
            raise Exception(f'AutoSummENG path "{autosummeng_root}" does not exist. Have you setup AutoSummENG?')

    def _save_summary(self, summary: SummaryType, file_path: str) -> None:
        dirname = os.path.dirname(file_path)
        if dirname:
            os.makedirs(dirname, exist_ok=True)

        with open(file_path, 'w') as out:
            if isinstance(summary, list):
                out.write('\n'.join(summary))
            else:
                out.write(summary)

    def _parse_output_file(self, file_path: str) -> List[List[MetricsDict]]:
        metrics_dicts = defaultdict(dict)
        with open(file_path, 'r') as f:
            for i, line in enumerate(f):
                # Header
                if i == 0:
                    continue
                columns = line.split('\t')
                if len(columns) != 5:
                    raise Exception(f'Expected 5 columns: {line}')

                instance_index = int(columns[0])
                summarizer_index = int(columns[1])
                metrics_dicts[instance_index][summarizer_index] = MetricsDict({
                    'AutoSummENG': float(columns[2]),
                    'MeMoG': float(columns[3]),
                    'NPowER': float(columns[4])
                })

        metrics_lists = []
        for i in range(len(metrics_dicts)):
            metrics_lists.append([])
            for j in range(len(metrics_dicts[i])):
                metrics_lists[-1].append(metrics_dicts[i][j])
        return metrics_lists

    def _run(self,
             summaries_list: List[List[SummaryType]],
             references_list: List[List[SummaryType]]) -> List[List[MetricsDict]]:
        with TemporaryDirectory() as temp_dir:
            files_tsv_path = f'{temp_dir}/files.tsv'
            with open(files_tsv_path, 'w') as out:
                for i, (summaries, references) in enumerate(zip(summaries_list, references_list)):
                    reference_filenames = []
                    for j, reference in enumerate(references):
                        filename = f'{temp_dir}/references/{i}/{j}.txt'
                        self._save_summary(reference, filename)
                        reference_filenames.append(filename)

                    peer_filenames = []
                    for j, summary in enumerate(summaries):
                        filename = f'{temp_dir}/peers/{i}/{j}.txt'
                        self._save_summary(summary, filename)
                        peer_filenames.append(filename)

                    out.write(f'{",".join(reference_filenames)}\t{",".join(peer_filenames)}\n')

            output_file = f'{temp_dir}/output.tsv'
            args = ' '.join([
                f'-files={files_tsv_path}',
                f'-output={output_file}',
                f'-minN={self.min_n}',
                f'-maxN={self.max_n}',
                f'-dwin={self.d_window}',
                f'-minScore={self.min_score}',
                f'-maxScore={self.max_score}'
            ])

            commands = [
                f'cd {self.autosummeng_root}',
                f'mvn exec:java@NPowERBatch -Dexec.args=\'{args}\''
            ]
            command = ' && '.join(commands)

            logger.info(f'Running AutoSummENG command: "{command}"')
            redirect = None if self.verbose else PIPE
            process = Popen(command, stdout=redirect, stderr=redirect, shell=True)
            stdout, stderr = process.communicate()

            return self._parse_output_file(output_file)

    def score_multi_all(self,
                        summaries_list: List[List[SummaryType]],
                        references_list: List[List[ReferenceType]]) -> List[List[MetricsDict]]:
        return self._run(summaries_list, references_list)


@MetricSetupSubcommand.register('autosummeng')
class AutoSummENGSetupSubcommand(MetricSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the AutoSummENG, MeMoG, and NPowER metrics'
        self.parser = parser.add_parser('autosummeng', description=description, help=description)
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        assert command_exists('mvn'), 'AutoSummENG requires Maven to be installed'

        commands = [
            f'mkdir -p {DATA_ROOT}/metrics',
            f'cd {DATA_ROOT}/metrics',
            f'git clone https://github.com/danieldeutsch/AutoSummENG',
            f'cd AutoSummENG',
            f'mvn package'
        ]
        command = ' && '.join(commands)

        process = Popen(command, shell=True)
        process.communicate()

        if process.returncode == 0:
            print('AutoSummENG setup success')
        else:
            print('AutoSummENG setup failure')



import unittest
from collections import defaultdict
from subprocess import PIPE, Popen

from sacrerouge.common import TemporaryDirectory
from sacrerouge.common.testing import FIXTURES_ROOT, MULTILING_SUMMARIES
from sacrerouge.common.testing.util import sacrerouge_command_exists
from sacrerouge.data import Metrics
from sacrerouge.io import JsonlReader

_config_file_path = f'{FIXTURES_ROOT}/configs/score.json'
_numeric_config_file_path = f'{FIXTURES_ROOT}/configs/evaluate-numeric.json'


class TestScore(unittest.TestCase):
    def test_command_exists(self):
        assert sacrerouge_command_exists(['score'])

    def test_score(self):
        with TemporaryDirectory() as temp_dir:
            # Runs a regression test for the "score" command
            output_file = f'{temp_dir}/metrics.jsonl'
            command = [
                'python', '-m', 'sacrerouge', 'score',
                '--config', _config_file_path,
                '--output-jsonl', output_file
            ]

            process = Popen(command, stdout=PIPE, stderr=PIPE)
            process.communicate()

            instances = JsonlReader(MULTILING_SUMMARIES).read()
            metrics_list = JsonlReader(output_file, Metrics).read()
            metrics_dicts = defaultdict(dict)

            assert len(instances) == len(metrics_list)
            for instance, metrics in zip(instances, metrics_list):
                assert metrics.instance_id == instance['instance_id']
                assert metrics.summarizer_id == instance['summarizer_id']
                assert metrics.summarizer_type == instance['summarizer_type']
                metrics_dicts[metrics.instance_id][metrics.summarizer_id] = metrics

                if metrics.summarizer_type == 'reference':
                    assert len(metrics.metrics) == 2
                    assert 'python-rouge-1_jk' in metrics.metrics
                    assert 'python-rouge-2_jk' in metrics.metrics
                else:
                    assert len(metrics.metrics) == 4
                    assert 'python-rouge-1' in metrics.metrics
                    assert 'python-rouge-2' in metrics.metrics
                    assert 'python-rouge-1_jk' in metrics.metrics
                    assert 'python-rouge-2_jk' in metrics.metrics

            # Test a couple of instances. I did not check to see if these are correct,
            # but the test will check if the results have changed
            assert metrics_dicts['M000']['1'].metrics == {
                "python-rouge-1": {
                  "precision": 41.699867197875164,
                  "recall": 40.516129032258064,
                  "f1": 41.09947643979057
                },
                "python-rouge-2": {
                  "precision": 10.533333333333333,
                  "recall": 10.233160621761659,
                  "f1": 10.38107752956636
                },
                "python-rouge-1_jk": {
                  "precision": 41.699867197875164,
                  "recall": 40.514662613316766,
                  "f1": 41.098355761265616
                },
                "python-rouge-2_jk": {
                  "precision": 10.533333333333333,
                  "recall": 10.226158358122346,
                  "f1": 10.3773782079838
                }
            }

            assert metrics_dicts['M001']['B'].metrics == {
                "python-rouge-1_jk": {
                  "precision": 51.59362549800797,
                  "recall": 51.18577075098815,
                  "f1": 51.3888888888889
                },
                "python-rouge-2_jk": {
                  "precision": 20.4,
                  "recall": 20.238095238095237,
                  "f1": 20.318725099601597
                }
            }

    def test_numeric_metric(self):
        with TemporaryDirectory() as temp_dir:
            output_file = f'{temp_dir}/metrics.jsonl'
            command = [
                'python', '-m', 'sacrerouge', 'score',
                '--config', _numeric_config_file_path,
                '--output-jsonl', output_file
            ]

            process = Popen(command, stdout=PIPE, stderr=PIPE)
            process.communicate()

            metrics_list = JsonlReader(output_file, Metrics).read()

            assert len(metrics_list) == 5
            assert metrics_list[0].instance_id == 'D1'
            assert metrics_list[1].instance_id == 'D1'
            assert metrics_list[2].instance_id == 'D1'
            assert metrics_list[3].instance_id == 'D1'
            assert metrics_list[4].instance_id == 'D1'

            assert metrics_list[0].summarizer_id == '1'
            assert metrics_list[1].summarizer_id == '2'
            assert metrics_list[2].summarizer_id == 'A'
            assert metrics_list[3].summarizer_id == 'B'
            assert metrics_list[4].summarizer_id == 'C'

            assert metrics_list[0].summarizer_type == 'peer'
            assert metrics_list[1].summarizer_type == 'peer'
            assert metrics_list[2].summarizer_type == 'reference'
            assert metrics_list[3].summarizer_type == 'reference'
            assert metrics_list[4].summarizer_type == 'reference'

            # test: 1 * 10 + 1 * 100 + 1 * 1000 == 1110
            # test_jk = ((1 * 10 + 1 * 100) + (1 * 10 + 1 * 1000) + (1 * 100 + 1 * 1000)) / 3 == 740
            assert metrics_list[0].metrics == {'test': 1110, 'test_jk': 740}
            # test: 2 * 10 + 2 * 100 + 2 * 1000 == 2220
            # test_jk = ((2 * 10 + 2 * 100) + (2 * 10 + 2 * 1000) + (2 * 100 + 2 * 1000)) / 3 == 1480
            assert metrics_list[1].metrics == {'test': 2220, 'test_jk': 1480}  # 2 * 10 + 2 * 100 + 2 * 1000
            # test_jk = 10 * 100 + 10 * 1000 == 11000
            assert metrics_list[2].metrics == {'test_jk': 11000}  # 10 * 100 + 10 * 1000
            # test_jk = 100 * 10 + 100 * 1000 == 101000
            assert metrics_list[3].metrics == {'test_jk': 101000}  # 100 * 10 + 100 * 1000
            # test_jk = 1000 * 10 + 10000 * 100 == 110000
            assert metrics_list[4].metrics == {'test_jk': 110000}  # 1000 * 10 + 10000 * 100

    def test_disable_jackknifing(self):
        with TemporaryDirectory() as temp_dir:
            # Runs a regression test for the "score" command
            output_file = f'{temp_dir}/metrics.jsonl'
            command = [
                'python', '-m', 'sacrerouge', 'score',
                '--config', _config_file_path,
                '--output-jsonl', output_file,
                '--disable-peer-jackknifing'
            ]

            process = Popen(command, stdout=PIPE, stderr=PIPE)
            process.communicate()

            instances = JsonlReader(MULTILING_SUMMARIES).read()
            metrics_list = JsonlReader(output_file, Metrics).read()
            metrics_dicts = defaultdict(dict)

            assert len(instances) == len(metrics_list)
            for instance, metrics in zip(instances, metrics_list):
                assert metrics.instance_id == instance['instance_id']
                assert metrics.summarizer_id == instance['summarizer_id']
                assert metrics.summarizer_type == instance['summarizer_type']
                metrics_dicts[metrics.instance_id][metrics.summarizer_id] = metrics

                if metrics.summarizer_type == 'reference':
                    assert len(metrics.metrics) == 2
                    assert 'python-rouge-1_jk' in metrics.metrics
                    assert 'python-rouge-2_jk' in metrics.metrics
                else:
                    assert len(metrics.metrics) == 2
                    assert 'python-rouge-1' in metrics.metrics
                    assert 'python-rouge-2' in metrics.metrics

            # Test a couple of instances. I did not check to see if these are correct,
            # but the test will check if the results have changed
            assert metrics_dicts['M000']['1'].metrics == {
                "python-rouge-1": {
                  "precision": 41.699867197875164,
                  "recall": 40.516129032258064,
                  "f1": 41.09947643979057
                },
                "python-rouge-2": {
                  "precision": 10.533333333333333,
                  "recall": 10.233160621761659,
                  "f1": 10.38107752956636
                }
            }

            assert metrics_dicts['M001']['B'].metrics == {
                "python-rouge-1_jk": {
                  "precision": 51.59362549800797,
                  "recall": 51.18577075098815,
                  "f1": 51.3888888888889
                },
                "python-rouge-2_jk": {
                  "precision": 20.4,
                  "recall": 20.238095238095237,
                  "f1": 20.318725099601597
                }
            }


import argparse
import json
import logging
import os
from overrides import overrides
from subprocess import Popen, PIPE
from typing import List

from sacrerouge.commands import MetricSetupSubcommand
from sacrerouge.common import DATA_ROOT, TemporaryDirectory
from sacrerouge.data import MetricsDict
from sacrerouge.data.types import DocumentType, SummaryType
from sacrerouge.metrics import DocumentBasedMetric, Metric

logger = logging.getLogger(__name__)


@Metric.register('supert')
class SUPERT(DocumentBasedMetric):
    def __init__(self,
                 environment_name: str = None,
                 supert_root: str = f'{DATA_ROOT}/metrics/SUPERT',
                 verbose: bool = False):
        super().__init__(['summary'], ['documents'])
        self.environment_name = environment_name
        self.supert_root = supert_root
        self.verbose = verbose

        if self.environment_name is not None:
            if 'CONDA_INIT' not in os.environ:
                raise Exception('If `environment_name` is not none, environment variable "CONDA_INIT" must be set to the path to "conda.sh"')

    def _save_documents(self, documents: List[List[DocumentType]], output_dir: str) -> None:
        os.makedirs(output_dir)
        for i, document in enumerate(documents):
            with open(f'{output_dir}/{i}.txt', 'w') as out:
                if isinstance(document, list):
                    document = ' '.join(document)
                out.write('<TEXT>\n')
                out.write(document + '\n')
                out.write('</TEXT>\n')

    def _save_summaries(self, summaries: List[List[SummaryType]], output_dir: str) -> None:
        os.makedirs(output_dir)
        for i, summary in enumerate(summaries):
            with open(f'{output_dir}/{i}', 'w') as out:
                if isinstance(summary, list):
                    summary = ' '.join(summary)
                out.write(summary)

    def score_multi_all(self,
                        summaries_list: List[List[SummaryType]],
                        documents_list: List[List[DocumentType]],
                        **kwargs) -> List[List[MetricsDict]]:
        with TemporaryDirectory() as temp_dir:
            input_dir = f'{temp_dir}/input'
            output_file = f'{temp_dir}/output.json'

            logger.info(f'Serializing data to {input_dir}')
            os.makedirs(input_dir)
            for i, (summaries, documents) in enumerate(zip(summaries_list, documents_list)):
                instance_dir = f'{input_dir}/{i}'
                documents_dir = f'{instance_dir}/input_docs'
                summaries_dir = f'{instance_dir}/summaries'

                self._save_documents(documents, documents_dir)
                self._save_summaries(summaries, summaries_dir)

            commands = [f'cd {self.supert_root}']
            if self.environment_name is not None:
                commands.append(f'source {os.environ["CONDA_INIT"]}')
                commands.append(f'conda activate {self.environment_name}')
            commands.append(f'python run_batch.py {input_dir} {output_file}')
            command = ' && '.join(commands)

            logger.info(f'Running command: "{command}"')
            redirect = None if self.verbose else PIPE
            process = Popen(command, stdout=redirect, stderr=redirect, shell=True)
            process.communicate()

            logger.info(f'Loading output from {output_file}')
            output = json.loads(open(output_file, 'r').read())
            metrics_list = []
            for i, summaries in enumerate(summaries_list):
                metrics_list.append([])
                for j in range(len(summaries)):
                    score = output[str(i)][str(j)]
                    # SUPERT will output None if the summary was empty, so we replace that with a 0.0
                    if score is None:
                        score = 0.0
                    metrics_list[-1].append(MetricsDict({'supert': score}))
            return metrics_list


@MetricSetupSubcommand.register('supert')
class SUPERTSetupSubcommand(MetricSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the SUPERT metric'
        self.parser = parser.add_parser('supert', description=description, help=description)
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        commands = [
            f'mkdir -p {DATA_ROOT}/metrics',
            f'cd {DATA_ROOT}/metrics',
            f'git clone https://github.com/danieldeutsch/SUPERT',
        ]
        command = ' && '.join(commands)

        process = Popen(command, shell=True)
        process.communicate()

        if process.returncode == 0:
            print('SUPERT setup success')
        else:
            print('SUPERT setup failure')


import unittest

from sacrerouge.common.util import command_exists


class TestUtil(unittest.TestCase):
    def test_command_exists(self):
        assert command_exists('python')
        assert not command_exists('asdfasd')



import argparse
import lxml.html
import re
import tarfile
from collections import defaultdict
from io import BytesIO
from nltk.tokenize import sent_tokenize
from typing import Any, Dict, List

from sacrerouge.io import JsonlWriter


def load_documents(documents_tar_path: str):
    clusters = defaultdict(list)
    documents = {}
    selectors = {}

    with tarfile.open(documents_tar_path, 'r') as tar:
        inner_tar_bytes = tar.extractfile('DUC2005_Summarization_Documents/duc2005_docs.tar.gz').read()
        with tarfile.open(fileobj=BytesIO(inner_tar_bytes)) as inner_tar:
            for member in inner_tar.getmembers():
                if member.isfile():
                    path = member.name.split('/')
                    cluster_id = path[1][:-1]
                    filename = path[2]
                    selector = path[1][-1].upper()
                    clusters[cluster_id].append(filename)
                    if cluster_id not in selectors:
                        selectors[cluster_id] = selector

                    html = inner_tar.extractfile(member).read().decode()
                    tree = lxml.html.document_fromstring(html)
                    text_nodes = list(tree.xpath('//text'))
                    if len(text_nodes) == 0:
                        text_nodes = list(tree.xpath('//graphic'))
                    assert len(text_nodes) == 1

                    text = text_nodes[0].text_content().strip()
                    text = re.sub('\s+', ' ', text)
                    sentences = sent_tokenize(text)
                    documents[filename] = {
                        'filename': filename,
                        'text': sentences
                    }

    return clusters, documents, selectors


def load_summaries(results_tar: str):
    summaries = defaultdict(list)
    with tarfile.open(results_tar, 'r') as tar:
        for member in tar.getmembers():
            if member.name.startswith('results/ROUGE/models/'):
                path = member.name.split('/')
                cluster = path[-1].split('.')[0].lower()
                annotator = path[-1].split('.')[-1]
                sentences = tar.extractfile(member).read().decode(errors='replace').splitlines()
                summary = {
                    'annotator': annotator,
                    'text': sentences
                }
                if summary not in summaries[cluster]:
                    summaries[cluster].append(summary)
    return summaries


def load_topics(topics_file_path: str):
    tree = lxml.html.parse(topics_file_path)
    nodes = list(tree.xpath('//topic'))

    topics = {}
    for node in nodes:
        cluster = node.xpath('num')[0].text_content().strip()[:-1]
        title = node.xpath('title')[0].text_content().strip()

        narr = node.xpath('narr')[0].text_content().strip()
        narr = re.sub('\s+', ' ', narr)
        narr = sent_tokenize(narr)

        granularity = node.xpath('granularity')[0].text_content().strip()
        topics[cluster] = {
            'title': title,
            'narrative': narr,
            'granularity': granularity
        }

    return topics


def save_data(clusters: Dict[str, List[str]],
              documents: Dict[str, List[str]],
              summaries: Dict[str, List[List[str]]],
              topics: Dict[str, Dict[str, Any]],
              selectors: Dict[str, str],
              file_path: str):
    with JsonlWriter(file_path) as out:
        for cluster, doc_ids in clusters.items():
            cluster_docs = [documents[doc_id] for doc_id in doc_ids]
            cluster_summaries = summaries[cluster]
            cluster_topic = topics[cluster]
            cluster_selector = selectors[cluster]
            data = {
                'id': cluster,
                'selector': cluster_selector,
                'topic': cluster_topic,
                'documents': cluster_docs,
                'summaries': cluster_summaries
            }
            if len(cluster_summaries) != 4:
                print(f'Found {len(cluster_summaries)} summaries for cluster {cluster}. Expected 4')
            out.write(data)


def setup(data_root: str, output_dir: str) -> None:
    documents_tar = f'{data_root}/from-nist/DUC2005_Summarization_Documents.tgz'
    results_tar = f'{data_root}/scrapes/duc.nist.gov/past_duc/duc2005/results/NIST/results.tar'
    topics_file_path = f'{data_root}/scrapes/duc.nist.gov/past_duc/duc2005/testdata/duc2005_topics.sgml'
    main(documents_tar, results_tar, topics_file_path, output_dir)


def main(documents_tar, results_tar, topics_file_path, output_dir):
    clusters, documents, selectors = load_documents(documents_tar)
    summaries = load_summaries(results_tar)
    topics = load_topics(topics_file_path)
    save_data(clusters, documents, summaries, topics, selectors, f'{output_dir}/task1.jsonl')


if __name__ == '__main__':
    argp = argparse.ArgumentParser()
    argp.add_argument('documents_tar')
    argp.add_argument('results_tar')
    argp.add_argument('topics_file_path')
    argp.add_argument('output_dir')
    args = argp.parse_args()

    main(args.documents_tar, args.results_tar, args.topics_file_path, args.output_dir)



import unittest
from typing import Dict, List

from sacrerouge.common import FromParams, Params, Registrable


class _Base1(Registrable):
    pass


@_Base1.register('subclass1')
class _Subclass1(_Base1):
    def __init__(self, a: int, b: str):
        self.a = a
        self.b = b


@_Base1.register('subclass2')
class _Subclass2(_Base1):
    def __init__(self,
                 first: List[_Base1],
                 second: Dict[str, _Base1]):
        self.first = first
        self.second = second


class TestFromParams(unittest.TestCase):
    def test_from_params(self):
        params = Params({
            'type': 'subclass1',
            'a': 4,
            'b': 'abc'
        })
        obj = _Base1.from_params(params)
        target = _Subclass1(4, 'abc')

        assert isinstance(obj, _Subclass1)
        assert obj.a == target.a
        assert obj.b == target.b

    def test_from_params_complex_types(self):
        params = Params({
            'type': 'subclass2',
            'first': [
                {'type': 'subclass1', 'a': 4, 'b': 'abc'},
                {'type': 'subclass1', 'a': 8, 'b': 'def'},
            ],
            'second': {
                'A': {'type': 'subclass1', 'a': 7, 'b': 'ghi'},
                'B': {'type': 'subclass1', 'a': 3, 'b': 'jkl'},
            }
        })
        obj = _Base1.from_params(params)
        target = _Subclass2(
            [
                _Subclass1(4, 'abc'),
                _Subclass1(8, 'def')
            ],
            {
                'A': _Subclass1(7, 'ghi'),
                'B': _Subclass1(3, 'jkl')
            }
        )

        assert isinstance(obj, _Subclass2)
        assert len(obj.first) == 2
        assert obj.first[0].a == 4
        assert obj.first[0].b == 'abc'
        assert obj.first[1].a == 8
        assert obj.first[1].b == 'def'

        assert len(obj.second) == 2
        assert obj.second['A'].a == 7
        assert obj.second['A'].b == 'ghi'
        assert obj.second['B'].a == 3
        assert obj.second['B'].b == 'jkl'



from sacrerouge import build_argument_parser


def main():
    parser = build_argument_parser()
    args = parser.parse_args()
    if 'func' in dir(args):
        args.func(args)
    else:
        parser.print_help()


if __name__ == '__main__':
    main()



import unittest

from sacrerouge.common import Registrable


class TestRegistrable(unittest.TestCase):
    class _Base1(Registrable):
        pass

    @_Base1.register('subclass1')
    class _Subclass1(_Base1):
        pass

    class _Base2(Registrable):
        pass

    @_Base2.register('subclass2')
    class _Subclass2(_Base2):
        pass

    def test_register(self):
        assert TestRegistrable._Base1 in Registrable._registry
        assert len(Registrable._registry[TestRegistrable._Base1]) == 1
        assert Registrable._registry[TestRegistrable._Base1]['subclass1'] == (TestRegistrable._Subclass1, None)

        assert TestRegistrable._Base2 in Registrable._registry
        assert len(Registrable._registry[TestRegistrable._Base2]) == 1
        assert Registrable._registry[TestRegistrable._Base2]['subclass2'] == (TestRegistrable._Subclass2, None)



FIXTURES_ROOT = 'sacrerouge/tests/fixtures'

MULTILING_DOCUMENTS = f'{FIXTURES_ROOT}/data/multiling2011/documents.jsonl'
MULTILING_SUMMARIES = f'{FIXTURES_ROOT}/data/multiling2011/summaries.jsonl'
MULTILING_METRICS = f'{FIXTURES_ROOT}/data/multiling2011/metrics.jsonl'

# Import after constants so the classes can use them
from sacrerouge.common.testing.metric_test_cases import ReferenceBasedMetricTestCase
from sacrerouge.common.testing.testing_metric import TestingMetric


import argparse
import logging
import os
import shutil
from collections import defaultdict
from overrides import overrides
from subprocess import Popen, PIPE
from typing import List, Tuple

from sacrerouge.commands import MetricSetupSubcommand
from sacrerouge.common import DATA_ROOT, TemporaryDirectory
from sacrerouge.common.util import command_exists, download_file_from_google_drive
from sacrerouge.data import MetricsDict
from sacrerouge.data.jackknifers import ReferencesJackknifer
from sacrerouge.data.types import ReferenceType, SummaryType
from sacrerouge.metrics import Metric, ReferenceBasedMetric

logger = logging.getLogger(__name__)


@Metric.register('bewte')
class BEwTE(ReferenceBasedMetric):
    def __init__(self,
                 bewte_root: str = f'{DATA_ROOT}/metrics/ROUGE-BEwTE',
                 verbose: bool = False):
        super().__init__(['summary'], ['references'], jackknifer=ReferencesJackknifer())
        self.bewte_root = bewte_root
        if not os.path.exists(bewte_root):
            raise Exception('BEwTE directory does not exist. Please run the setup code')
        self.pos_model = f'src/main/resources/models/posTaggingModel.gz'
        self.parsing_model = f'src/main/resources/models/parseModel.gz'
        self.wordnet_dir = f'src/main/resources/data/wordnet3_0'
        self.opennlp_dir = f'src/main/resources/models/opennlp'
        self.rule_file = f'src/main/resources/conf/rules/EN_ruleList.txt'
        self.transforms_file = f'src/main/resources/conf/transformations/EN_transformsList.txt'
        self.transforms_coef_file = f'src/main/resources/conf/transformations/EN_transformCoeffs.txt'
        self.end_analysis_file = f'src/main/resources/conf/endanalysis/doNothingEndAnalysisConfig.txt'
        self.verbose = verbose

    def _save_summary(self, summary: SummaryType, file_path: str) -> None:
        dirname = os.path.dirname(file_path)
        os.makedirs(dirname, exist_ok=True)
        with open(file_path, 'w') as out:
            if isinstance(summary, list):
                out.write('\n'.join(summary))
            else:
                out.write(summary)

    def _save_summaries(self,
                        temp_dir: str,
                        summaries_list: List[List[SummaryType]],
                        references_list: List[List[SummaryType]]) -> None:
        for i, (summaries, references) in enumerate(zip(summaries_list, references_list)):
            for j, summary in enumerate(summaries):
                self._save_summary(summary, f'{temp_dir}/summaries/{i}.{j}')
            for j, reference in enumerate(references):
                symbol = chr(j + 65)
                self._save_summary(reference, f'{temp_dir}/summaries/{i}.{symbol}')

    def _run_step1(self, temp_dir: str) -> None:
        args = ' '.join([
            '-corpusreader',
            'tratz.runpipe.impl.corpusreader.DirectoryCorpusReader',
            f'InputDirectories={temp_dir}/summaries',
            '-documentreader',
            'bewte.io.StandardTextDocReader',
            '-annotator',
            'tratz.runpipe.impl.annotators.sentence.BreakIteratorSentenceAnnotator',
            'ONLY_WHEN_NECESSARY=true',
            '-annotator',
            'bewte.annotators.RegexTokenizer',
            '-annotator',
            'tratz.runpipe.impl.annotators.pos.TratzPosTaggerAnnotator',
            f'ModelFile={self.pos_model}',
            f'WordNetDir={self.wordnet_dir}',
            '-annotator',
            'tratz.runpipe.impl.annotators.parse.TratzParserAnnotator',
            f'ModelFile={self.parsing_model}',
            f'WordNetDir={self.wordnet_dir}',
            'VchTransform=true',
            '-endpoint',
            'tratz.runpipe.impl.endpoints.GzippedDocumentWriter',
            f'OutputDir={temp_dir}/eval/temp/parsed'
        ])

        commands = [
            f'cd {self.bewte_root}',
            f'mvn exec:java@RunPipe -Dexec.args=\'{args}\''
        ]
        command = ' && '.join(commands)

        logger.info(f'Running BEwTE step 1 command: "{command}"')
        redirect = None if self.verbose else PIPE
        process = Popen(command, stdout=redirect, stderr=redirect, shell=True)
        process.communicate()

    def _run_step2(self, temp_dir: str) -> None:
        args = ' '.join([
            '-corpusreader',
            'tratz.runpipe.impl.corpusreader.GzippedCorpusReader',
            f'InputDirectories={temp_dir}/eval/temp/parsed',
            '-annotator',
            'tratz.runpipe.impl.annotators.parse.TokenFieldUpdater',
            f'WordNetDir={self.wordnet_dir}',
            '-annotator',
            'runpipewrappers.ner.OpenNlpNerWrapper',
            f'ModelPath={self.opennlp_dir}/person.bin.gz',
            'AnnotationClass=tratz.runpipe.annotations.PersonAnnotation',
            '-annotator',
            'runpipewrappers.ner.OpenNlpNerWrapper',
            f'ModelPath={self.opennlp_dir}/organization.bin.gz',
            'AnnotationClass=tratz.runpipe.annotations.OrganizationAnnotation',
            '-annotator',
            'runpipewrappers.ner.OpenNlpNerWrapper',
            f'ModelPath={self.opennlp_dir}/location.bin.gz',
            'AnnotationClass=tratz.runpipe.annotations.LocationAnnotation',
            '-endpoint',
            'bewte.beextraction.BasicElementExtractor',
            f'OutputDir={temp_dir}/eval/temp/BEs'
        ])

        commands = [
            f'cd {self.bewte_root}',
            f'mvn exec:java@RunPipe -Dexec.args=\'{args}\''
        ]
        command = ' && '.join(commands)

        logger.info(f'Running BEwTE step 2 command: "{command}"')
        redirect = None if self.verbose else PIPE
        process = Popen(command, stdout=redirect, stderr=redirect, shell=True)
        process.communicate()

    def _run_step3(self, temp_dir: str) -> None:
        args = ' '.join([
            f'{temp_dir}/eval/temp/BEs',
            f'{temp_dir}/eval/temp/BEXs',
            self.wordnet_dir,
            '0',
            '-1',
            '.*[A-Z_\\-]+',
            self.transforms_file,
            'bewte.names.DUCStyleNameExtractor'
        ])

        commands = [
            f'cd {self.bewte_root}',
            f'mvn exec:java@BEXpander -Dexec.args=\'{args}\''
        ]
        command = ' && '.join(commands)

        logger.info(f'Running BEwTE step 3 command: "{command}"')
        redirect = None if self.verbose else PIPE
        process = Popen(command, stdout=redirect, stderr=redirect, shell=True)
        process.communicate()

    def _run_step4(self, temp_dir: str) -> None:
        args = ' '.join([
            f'{temp_dir}/eval/temp/BEXs',
            f'{temp_dir}/systemLevelOutput.txt',
            f'{temp_dir}/summaryLevelOutput.txt',
            'bewte.scoring.TallyFunction$BinaryTallyFunction',
            'false',
            self.rule_file,
            self.transforms_file,
            self.transforms_coef_file,
            '[A-Z_\\-]+',
            self.end_analysis_file,
            'bewte.names.DUCStyleNameExtractor',
            '.*'
        ])

        commands = [
            f'cd {self.bewte_root}',
            f'mvn exec:java@BEwT_E -Dexec.args=\'{args}\''
        ]
        command = ' && '.join(commands)

        logger.info(f'Running BEwTE step 4 command: "{command}"')
        process = Popen(command, stdout=PIPE, stderr=PIPE, shell=True)
        stdout, _ = process.communicate()
        return stdout.decode()

    def _get_topic_and_system(self, line: str) -> Tuple[int, str]:
        last_slash = line.rfind('/')
        last_period = line.rfind('.')

        topic = int(line[last_slash + 1:last_period])
        system = line[last_period + 1:]
        return topic, system

    def _get_f1(self, line: str) -> Tuple[float, float, float]:
        columns = line.split()
        precision = float(columns[3][2:]) * 100
        recall = float(columns[4][2:]) * 100
        if precision + recall == 0:
            f1 = 0.0
        else:
            f1 = 2 * (precision * recall) / (precision + recall)
        return precision, recall, f1

    def _parse_stdout(self, stdout: str) -> List[List[MetricsDict]]:
        lines = stdout.splitlines()
        metrics_dicts = defaultdict(lambda: defaultdict(MetricsDict))

        index = 0
        while index < len(lines):
            line = lines[index].strip()
            if line.startswith('Peer:'):
                topic, system = self._get_topic_and_system(line)
                if not system.isalpha():
                    system = int(system)
                    precisions, recalls, f1s = [], [], []
                    index += 2  # skip 'Fast score calc...'
                    while lines[index] != '...done':
                        precision, recall, f1 = self._get_f1(lines[index])
                        precisions.append(precision)
                        recalls.append(recall)
                        f1s.append(f1)
                        index += 1

                    metrics_dicts[topic][system]['BEwTE'] = {
                        'precision': sum(precisions) / len(precisions),
                        'recall': sum(recalls) / len(recalls),
                        'f1': sum(f1s) / len(f1s)
                    }
            index += 1

        metrics_lists = []
        for i in range(len(metrics_dicts)):
            metrics_list = []
            for j in range(len(metrics_dicts[i])):
                metrics_list.append(metrics_dicts[i][j])
            metrics_lists.append(metrics_list)
        return metrics_lists

    def score_multi_all(self,
                        summaries_list: List[List[SummaryType]],
                        references_list: List[List[ReferenceType]]) -> List[List[MetricsDict]]:
        with TemporaryDirectory() as temp_dir:
            self._save_summaries(temp_dir, summaries_list, references_list)

            self._run_step1(temp_dir)
            self._run_step2(temp_dir)
            self._run_step3(temp_dir)
            stdout = self._run_step4(temp_dir)

            # There is a weird way to score a summary given multiple references
            # in the original code. They multiply the highest recall score by
            # (num_references - 1) and adds that to the second to last score and
            # divides by num_references.
            # (See https://github.com/igorbrigadir/ROUGE-BEwTE/blob/f69a85556c889b805c89c5c71d7b77a983e75a05/src/main/java/bewte/BEwT_E.java#L419)
            # I don't understand this because it depends on the order that the
            # summaries are processed. We instead compute the average over the references.
            metrics_lists = self._parse_stdout(stdout)
            return metrics_lists


@MetricSetupSubcommand.register('bewte')
class BEwTESetupSubcommand(MetricSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the BEwT-E metric'
        self.parser = parser.add_parser('bewte', description=description, help=description)
        self.parser.add_argument('--force', action='store_true', help='Force setting up the metric again')
        self.parser.set_defaults(subfunc=self.run)

    def _edit_pom(self, file_path: str) -> None:
        # We need to edit the pom.xml file to add options to run the main classes
        lines = open(file_path, 'r').read().splitlines()
        with open(file_path, 'w') as out:
            for line in lines[:80]:
                out.write(line + '\n')
            out.write('''
                <plugin>
                    <groupId>org.codehaus.mojo</groupId>
                    <artifactId>exec-maven-plugin</artifactId>
                    <version>1.6.0</version>
                    <executions>
                      <execution>
                        <id>RunPipe</id>
                        <configuration>
                          <mainClass>tratz.runpipe.util.RunPipe</mainClass>
                        </configuration>
                      </execution>
                      <execution>
                        <id>BEXpander</id>
                        <configuration>
                          <mainClass>bewte.BEXpander</mainClass>
                        </configuration>
                      </execution>
                      <execution>
                        <id>BEwT_E</id>
                        <configuration>
                          <mainClass>bewte.BEwT_E</mainClass>
                        </configuration>
                      </execution>
                    </executions>
                </plugin>
            ''')
            for line in lines[80:]:
                out.write(line + '\n')

    @overrides
    def run(self, args):
        assert command_exists('mvn'), 'BEwTE requires Maven to be installed'

        if args.force and os.path.exists(f'{DATA_ROOT}/metrics/ROUGE-BEwTE'):
            shutil.rmtree(f'{DATA_ROOT}/metrics/ROUGE-BEwTE')

        # We have to clone the ROUGE-BEwTE repository and disable git-lfs, otherwise we may cause the repo
        # to exceed the bandwidth quota, which results in a cloning failure. Afterward, the model files are downloaded
        # and put into place
        commands = [
            f'mkdir -p {DATA_ROOT}/metrics',
            f'cd {DATA_ROOT}/metrics',
            f'GIT_LFS_SKIP_SMUDGE=1 git clone https://github.com/igorbrigadir/ROUGE-BEwTE',

        ]
        command = ' && '.join(commands)

        process = Popen(command, shell=True)
        process.communicate()
        if process.returncode != 0:
            print('BEwT-E setup failure')
            return

        # Download the models, unzip, and move to the correct directory
        download_file_from_google_drive('1d0DjP8sxNoro_9fXaAlhB5JgqHjWMgst', f'{DATA_ROOT}/metrics/ROUGE-BEwTE/models.zip')
        commands = [
            f'cd {DATA_ROOT}/metrics/ROUGE-BEwTE',
            f'unzip models.zip',
            f'rm models.zip',
            f'rm -r src/main/resources/models',
            f'mv models src/main/resources/'
        ]
        command = ' && '.join(commands)

        process = Popen(command, shell=True)
        process.communicate()
        if process.returncode != 0:
            print('BEwT-E setup failure')
            return

        self._edit_pom(f'{DATA_ROOT}/metrics/ROUGE-BEwTE/pom.xml')

        commands = [
            f'cd {DATA_ROOT}/metrics/ROUGE-BEwTE',
            f'mvn package'
        ]
        command = ' && '.join(commands)

        process = Popen(command, shell=True)
        process.communicate()
        if process.returncode == 0:
            print('BEwT-E setup success')
        else:
            print('BEwT-E setup failure')



import os
import pytest
import unittest

from sacrerouge.commands.correlate import aggregate_metrics
from sacrerouge.data import Metrics
from sacrerouge.io import JsonlReader

_metrics_A_file_path = 'datasets/duc-tac/tac2010/v1.0/task1.A.metrics.jsonl'
_metrics_B_file_path = 'datasets/duc-tac/tac2010/v1.0/task1.B.metrics.jsonl'


class TestTAC2010SystemLevel(unittest.TestCase):
    @pytest.mark.skipif(not os.path.exists(_metrics_A_file_path), reason='TAC 2010-A metrics file does not exist')
    def test_system_level_A(self):
        summary_level_metrics = JsonlReader(_metrics_A_file_path, Metrics).read()
        system_level_metrics = aggregate_metrics(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files
        # ROUGE/rouge2_A.m.avg
        assert system_level_metrics['22']['rouge-2']['recall'] == pytest.approx(9.574, 1e-2)
        assert system_level_metrics['18']['rouge-2']['recall'] == pytest.approx(9.418, 1e-2)
        assert system_level_metrics['23']['rouge-2']['recall'] == pytest.approx(9.404, 1e-2)
        assert system_level_metrics['24']['rouge-2']['recall'] == pytest.approx(9.196, 1e-2)
        assert system_level_metrics['36']['rouge-2']['recall'] == pytest.approx(9.194, 1e-2)

        # ROUGE/rouge2_A.jk.m.avg
        assert system_level_metrics['D']['rouge-2_jk']['recall'] == pytest.approx(12.862, 1e-2)
        assert system_level_metrics['H']['rouge-2_jk']['recall'] == pytest.approx(12.841, 1e-1)
        assert system_level_metrics['F']['rouge-2_jk']['recall'] == pytest.approx(12.556, 1e-2)
        assert system_level_metrics['22']['rouge-2_jk']['recall'] == pytest.approx(9.620, 1e-2)
        assert system_level_metrics['18']['rouge-2_jk']['recall'] == pytest.approx(9.451, 1e-2)

        # ROUGE/rougeSU4_A.m.avg
        assert system_level_metrics['22']['rouge-su4']['recall'] == pytest.approx(13.014, 1e-2)
        assert system_level_metrics['23']['rouge-su4']['recall'] == pytest.approx(12.963, 1e-2)
        assert system_level_metrics['24']['rouge-su4']['recall'] == pytest.approx(12.829, 1e-2)
        assert system_level_metrics['18']['rouge-su4']['recall'] == pytest.approx(12.407, 1e-2)
        assert system_level_metrics['34']['rouge-su4']['recall'] == pytest.approx(12.283, 1e-2)

        # ROUGE/rougeSU4_A.jk.m.avg
        assert system_level_metrics['H']['rouge-su4_jk']['recall'] == pytest.approx(16.294, 1e-2)
        assert system_level_metrics['F']['rouge-su4_jk']['recall'] == pytest.approx(16.212, 1e-2)
        assert system_level_metrics['D']['rouge-su4_jk']['recall'] == pytest.approx(16.200, 1e-2)
        assert system_level_metrics['22']['rouge-su4_jk']['recall'] == pytest.approx(13.049, 1e-2)
        assert system_level_metrics['23']['rouge-su4_jk']['recall'] == pytest.approx(12.978, 1e-2)

        # manual/manual.model.A.avg
        assert system_level_metrics['A']['num_scus_jk'] == pytest.approx(10.870, 1e-2)
        assert system_level_metrics['B']['num_scus_jk'] == pytest.approx(11.087, 1e-2)
        assert system_level_metrics['C']['num_scus_jk'] == pytest.approx(9.826, 1e-2)

        assert system_level_metrics['A']['modified_pyramid_score_jk'] == pytest.approx(0.779, 1e-2)
        assert system_level_metrics['B']['modified_pyramid_score_jk'] == pytest.approx(0.747, 1e-2)
        assert system_level_metrics['C']['modified_pyramid_score_jk'] == pytest.approx(0.661, 1e-2)

        assert system_level_metrics['A']['linguistic_quality'] == pytest.approx(4.913, 1e-2)
        assert system_level_metrics['B']['linguistic_quality'] == pytest.approx(4.870, 1e-2)
        assert system_level_metrics['C']['linguistic_quality'] == pytest.approx(4.826, 1e-2)

        assert system_level_metrics['A']['overall_responsiveness'] == pytest.approx(4.783, 1e-2)
        assert system_level_metrics['B']['overall_responsiveness'] == pytest.approx(4.696, 1e-2)
        assert system_level_metrics['C']['overall_responsiveness'] == pytest.approx(4.565, 1e-2)

        # manual/manual.peer.A.avg
        assert system_level_metrics['1']['modified_pyramid_score'] == pytest.approx(0.233, 1e-2)
        assert system_level_metrics['2']['modified_pyramid_score'] == pytest.approx(0.296, 1e-2)
        assert system_level_metrics['3']['modified_pyramid_score'] == pytest.approx(0.399, 1e-2)

        assert system_level_metrics['1']['num_scus'] == pytest.approx(3.304, 1e-2)
        assert system_level_metrics['2']['num_scus'] == pytest.approx(4.217, 1e-2)
        assert system_level_metrics['3']['num_scus'] == pytest.approx(5.500, 1e-2)

        assert system_level_metrics['1']['num_repetitions'] == pytest.approx(0.522, 1e-2)
        assert system_level_metrics['2']['num_repetitions'] == pytest.approx(1.217, 1e-2)
        assert system_level_metrics['3']['num_repetitions'] == pytest.approx(1.413, 1e-2)

        assert system_level_metrics['1']['modified_pyramid_score_jk'] == pytest.approx(0.229, 1e-2)
        assert system_level_metrics['2']['modified_pyramid_score_jk'] == pytest.approx(0.291, 1e-2)
        assert system_level_metrics['3']['modified_pyramid_score_jk'] == pytest.approx(0.393, 1e-2)

        assert system_level_metrics['1']['linguistic_quality'] == pytest.approx(3.652, 1e-2)
        assert system_level_metrics['2']['linguistic_quality'] == pytest.approx(2.717, 1e-2)
        assert system_level_metrics['3']['linguistic_quality'] == pytest.approx(3.043, 1e-2)

        assert system_level_metrics['1']['overall_responsiveness'] == pytest.approx(2.174, 1e-2)
        assert system_level_metrics['2']['overall_responsiveness'] == pytest.approx(2.500, 1e-2)
        assert system_level_metrics['3']['overall_responsiveness'] == pytest.approx(2.978, 1e-2)

        # BE/simple_A.m.hm.avg
        assert system_level_metrics['22']['rouge-be-hm']['recall'] == pytest.approx(5.937, 1e-2)
        assert system_level_metrics['23']['rouge-be-hm']['recall'] == pytest.approx(5.809, 1e-2)
        assert system_level_metrics['18']['rouge-be-hm']['recall'] == pytest.approx(5.749, 1e-2)
        assert system_level_metrics['13']['rouge-be-hm']['recall'] == pytest.approx(5.553, 1e-2)
        assert system_level_metrics['16']['rouge-be-hm']['recall'] == pytest.approx(5.497, 1e-2)

        # BE/simplejk_A.m.hm.avg
        assert system_level_metrics['F']['rouge-be-hm_jk']['recall'] == pytest.approx(9.114, 1e-2)
        assert system_level_metrics['H']['rouge-be-hm_jk']['recall'] == pytest.approx(8.690, 1e-1)
        assert system_level_metrics['D']['rouge-be-hm_jk']['recall'] == pytest.approx(8.449, 1e-1)
        assert system_level_metrics['22']['rouge-be-hm_jk']['recall'] == pytest.approx(5.973, 1e-2)
        assert system_level_metrics['23']['rouge-be-hm_jk']['recall'] == pytest.approx(5.828, 1e-2)

        # aesop_allpeers_A
        assert system_level_metrics['A']['aesop']['1'] == pytest.approx(0.09517478261, 1e-2)
        assert system_level_metrics['C']['aesop']['8'] == pytest.approx(0.0, 1e-2)
        assert system_level_metrics['4']['aesop']['13'] == pytest.approx(0.6150630435, 1e-2)
        assert system_level_metrics['8']['aesop']['22'] == pytest.approx(0.3684913043, 1e-2)
        assert system_level_metrics['16']['aesop']['27'] == pytest.approx(11.80434783, 1e-2)

    @pytest.mark.skipif(not os.path.exists(_metrics_B_file_path), reason='TAC 2010-B metrics file does not exist')
    def test_system_level_B(self):
        summary_level_metrics = JsonlReader(_metrics_B_file_path, Metrics).read()
        system_level_metrics = aggregate_metrics(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files
        # ROUGE/rouge2_B.m.avg
        assert system_level_metrics['16']['rouge-2']['recall'] == pytest.approx(8.024, 1e-2)
        assert system_level_metrics['13']['rouge-2']['recall'] == pytest.approx(7.913, 1e-2)
        assert system_level_metrics['36']['rouge-2']['recall'] == pytest.approx(7.311, 1e-2)
        assert system_level_metrics['8']['rouge-2']['recall'] == pytest.approx(7.251, 1e-2)
        assert system_level_metrics['4']['rouge-2']['recall'] == pytest.approx(7.058, 1e-2)

        # ROUGE/rouge2_B.jk.m.avg
        assert system_level_metrics['D']['rouge-2_jk']['recall'] == pytest.approx(13.021, 1e-2)
        assert system_level_metrics['E']['rouge-2_jk']['recall'] == pytest.approx(10.196, 1e-1)
        assert system_level_metrics['F']['rouge-2_jk']['recall'] == pytest.approx(9.777, 1e-2)
        assert system_level_metrics['16']['rouge-2_jk']['recall'] == pytest.approx(7.993, 1e-2)
        assert system_level_metrics['13']['rouge-2_jk']['recall'] == pytest.approx(7.902, 1e-2)

        # ROUGE/rougeSU4_B.m.avg
        assert system_level_metrics['16']['rouge-su4']['recall'] == pytest.approx(12.006, 1e-2)
        assert system_level_metrics['13']['rouge-su4']['recall'] == pytest.approx(11.878, 1e-2)
        assert system_level_metrics['6']['rouge-su4']['recall'] == pytest.approx(11.198, 1e-2)
        assert system_level_metrics['22']['rouge-su4']['recall'] == pytest.approx(11.107, 1e-2)
        assert system_level_metrics['8']['rouge-su4']['recall'] == pytest.approx(11.039, 1e-2)

        # ROUGE/rougeSU4_B.jk.m.avg
        assert system_level_metrics['D']['rouge-su4_jk']['recall'] == pytest.approx(16.193, 1e-2)
        assert system_level_metrics['E']['rouge-su4_jk']['recall'] == pytest.approx(13.978, 1e-2)
        assert system_level_metrics['G']['rouge-su4_jk']['recall'] == pytest.approx(13.573, 1e-2)
        assert system_level_metrics['16']['rouge-su4_jk']['recall'] == pytest.approx(11.979, 1e-2)
        assert system_level_metrics['13']['rouge-su4_jk']['recall'] == pytest.approx(11.869, 1e-2)

        # manual/manual.model.B.avg
        assert system_level_metrics['A']['num_scus_jk'] == pytest.approx(6.609, 1e-2)
        assert system_level_metrics['B']['num_scus_jk'] == pytest.approx(7.696, 1e-2)
        assert system_level_metrics['C']['num_scus_jk'] == pytest.approx(5.913, 1e-2)

        assert system_level_metrics['A']['modified_pyramid_score_jk'] == pytest.approx(0.629, 1e-2)
        assert system_level_metrics['B']['modified_pyramid_score_jk'] == pytest.approx(0.729, 1e-2)
        assert system_level_metrics['C']['modified_pyramid_score_jk'] == pytest.approx(0.551, 1e-2)

        assert system_level_metrics['A']['linguistic_quality'] == pytest.approx(4.913, 1e-2)
        assert system_level_metrics['B']['linguistic_quality'] == pytest.approx(4.826, 1e-2)
        assert system_level_metrics['C']['linguistic_quality'] == pytest.approx(4.870, 1e-2)

        assert system_level_metrics['A']['overall_responsiveness'] == pytest.approx(4.783, 1e-2)
        assert system_level_metrics['B']['overall_responsiveness'] == pytest.approx(4.783, 1e-2)
        assert system_level_metrics['C']['overall_responsiveness'] == pytest.approx(4.826, 1e-2)

        # manual/manual.peer.B.avg
        assert system_level_metrics['1']['modified_pyramid_score'] == pytest.approx(0.187, 1e-2)
        assert system_level_metrics['2']['modified_pyramid_score'] == pytest.approx(0.262, 1e-2)
        assert system_level_metrics['3']['modified_pyramid_score'] == pytest.approx(0.235, 1e-2)

        assert system_level_metrics['1']['num_scus'] == pytest.approx(2.065, 1e-2)
        assert system_level_metrics['2']['num_scus'] == pytest.approx(2.804, 1e-2)
        assert system_level_metrics['3']['num_scus'] == pytest.approx(2.609, 1e-2)

        assert system_level_metrics['1']['num_repetitions'] == pytest.approx(0.348, 1e-2)
        assert system_level_metrics['2']['num_repetitions'] == pytest.approx(0.522, 1e-2)
        assert system_level_metrics['3']['num_repetitions'] == pytest.approx(0.348, 1e-2)

        assert system_level_metrics['1']['modified_pyramid_score_jk'] == pytest.approx(0.184, 1e-2)
        assert system_level_metrics['2']['modified_pyramid_score_jk'] == pytest.approx(0.256, 1e-2)
        assert system_level_metrics['3']['modified_pyramid_score_jk'] == pytest.approx(0.228, 1e-2)

        assert system_level_metrics['1']['linguistic_quality'] == pytest.approx(3.739, 1e-2)
        assert system_level_metrics['2']['linguistic_quality'] == pytest.approx(2.696, 1e-2)
        assert system_level_metrics['3']['linguistic_quality'] == pytest.approx(2.957, 1e-2)

        assert system_level_metrics['1']['overall_responsiveness'] == pytest.approx(2.022, 1e-2)
        assert system_level_metrics['2']['overall_responsiveness'] == pytest.approx(2.478, 1e-2)
        assert system_level_metrics['3']['overall_responsiveness'] == pytest.approx(2.217, 1e-2)

        # BE/simple_B.m.hm.avg
        assert system_level_metrics['16']['rouge-be-hm']['recall'] == pytest.approx(4.445, 1e-2)
        assert system_level_metrics['13']['rouge-be-hm']['recall'] == pytest.approx(4.417, 1e-2)
        assert system_level_metrics['8']['rouge-be-hm']['recall'] == pytest.approx(4.350, 1e-1)
        assert system_level_metrics['4']['rouge-be-hm']['recall'] == pytest.approx(4.115, 1e-2)
        assert system_level_metrics['22']['rouge-be-hm']['recall'] == pytest.approx(4.050, 1e-2)

        # BE/simplejk_B.m.hm.avg
        assert system_level_metrics['D']['rouge-be-hm_jk']['recall'] == pytest.approx(8.842, 1e-2)
        assert system_level_metrics['F']['rouge-be-hm_jk']['recall'] == pytest.approx(7.842, 1e-1)
        assert system_level_metrics['B']['rouge-be-hm_jk']['recall'] == pytest.approx(7.081, 1e-1)
        assert system_level_metrics['16']['rouge-be-hm_jk']['recall'] == pytest.approx(4.411, 1e-2)
        assert system_level_metrics['13']['rouge-be-hm_jk']['recall'] == pytest.approx(4.402, 1e-2)

        # aesop_allpeers_B
        assert system_level_metrics['B']['aesop']['2'] == pytest.approx(0.1358091304, 1e-2)
        assert system_level_metrics['E']['aesop']['4'] == pytest.approx(0.1376682609, 1e-2)
        assert system_level_metrics['6']['aesop']['7'] == pytest.approx(0.2641304348, 1e-2)
        assert system_level_metrics['9']['aesop']['20'] == pytest.approx(0.09438347826, 1e-2)
        assert system_level_metrics['14']['aesop']['22'] == pytest.approx(0.3394478261, 1e-2)
